@@ -35,6 +35,7 @@
 #include "redis_slot.h"
 #include "server/redis_connection.h"
 #include "status.h"
+#include "storage/scripting.h"
 
 class ClusterNode {
  public:
@@ -47,7 +48,7 @@ class ClusterNode {
   std::string master_id;
   std::bitset<kClusterSlots> slots;
   std::vector<std::string> replicas;
-  int importing_slot = -1;
+  SlotRange importing_slot_range = {-1, -1};
 };
 
 struct SlotInfo {
@@ -74,8 +75,8 @@ class Cluster {
   StatusOr<std::string> GetReplicas(const std::string &node_id);
   Status SetNodeId(const std::string &node_id);
   Status SetSlotRanges(const std::vector<SlotRange> &slot_ranges, const std::string &node_id, int64_t version);
-  Status SetSlotMigrated(int slot, const std::string &ip_port);
-  Status SetSlotImported(int slot);
+  Status SetSlotRangeMigrated(const SlotRange &slot_range, const std::string &ip_port);
+  Status SetSlotRangeImported(const SlotRange &slot_range);
   Status GetSlotsInfo(std::vector<SlotInfo> *slot_infos);
   Status GetClusterInfo(std::string *cluster_infos);
   int64_t GetVersion() const { return version_; }
@@ -83,10 +84,11 @@ class Cluster {
   bool IsNotMaster();
   bool IsWriteForbiddenSlot(int slot) const;
   Status CanExecByMySelf(const redis::CommandAttributes *attributes, const std::vector<std::string> &cmd_tokens,
-                         redis::Connection *conn);
+                         redis::Connection *conn, lua::ScriptRunCtx *script_run_ctx = nullptr);
   Status SetMasterSlaveRepl();
-  Status MigrateSlot(int slot, const std::string &dst_node_id, SyncMigrateContext *blocking_ctx = nullptr);
-  Status ImportSlot(redis::Connection *conn, int slot, int state);
+  Status MigrateSlotRange(const SlotRange &slot_range, const std::string &dst_node_id,
+                          SyncMigrateContext *blocking_ctx = nullptr);
+  Status ImportSlotRange(redis::Connection *conn, const SlotRange &slot_range, int state);
   std::string GetMyId() const { return myid_; }
   Status DumpClusterNodes(const std::string &file);
   Status LoadClusterNodes(const std::string &file_path);
