@@ -35,18 +35,25 @@
 #include <vector>
 
 #include "batch_sender.h"
-#include "config.h"
 #include "encoding.h"
 #include "parse_util.h"
-#include "redis_slot.h"
 #include "server/server.h"
 #include "slot_import.h"
-#include "stats/stats.h"
 #include "status.h"
 #include "storage/redis_db.h"
 #include "unique_fd.h"
 
-enum class MigrationType { kRedisCommand = 0, kRawKeyValue };
+enum class MigrationType {
+  /// Use Redis commands to migrate data.
+  /// It will trying to extract commands from existing data and log, then replay
+  /// them on the destination node.
+  kRedisCommand = 0,
+  /// Using raw key-value and "APPLYBATCH" command in kvrocks to migrate data.
+  ///
+  /// If downstream is not compatible with raw key-value, this migration type will
+  /// auto switch to kRedisCommand.
+  kRawKeyValue
+};
 
 enum class MigrationState { kNone = 0, kStarted, kSuccess, kFailed };
 
@@ -55,8 +62,9 @@ enum class SlotMigrationStage { kNone, kStart, kSnapshot, kWAL, kSuccess, kFaile
 enum class KeyMigrationResult { kMigrated, kExpired, kUnderlyingStructEmpty };
 
 struct SlotMigrationJob {
-  SlotMigrationJob(int slot_id, std::string dst_ip, int dst_port, int speed, int pipeline_size, int seq_gap)
-      : slot_id(static_cast<int16_t>(slot_id)),
+  SlotMigrationJob(const SlotRange &slot_range_in, std::string dst_ip, int dst_port, int speed, int pipeline_size,
+                   int seq_gap)
+      : slot_range(slot_range_in),
         dst_ip(std::move(dst_ip)),
         dst_port(dst_port),
         max_speed(speed),
@@ -66,7 +74,7 @@ struct SlotMigrationJob {
   SlotMigrationJob &operator=(const SlotMigrationJob &other) = delete;
   ~SlotMigrationJob() = default;
 
-  int16_t slot_id;
+  SlotRange slot_range;
   std::string dst_ip;
   int dst_port;
   int max_speed;
@@ -84,9 +92,9 @@ class SlotMigrator : public redis::Database {
   ~SlotMigrator();
 
   Status CreateMigrationThread();
-  Status PerformSlotMigration(const std::string &node_id, std::string &dst_ip, int dst_port, int slot_id,
-                              SyncMigrateContext *blocking_ctx = nullptr);
-  void ReleaseForbiddenSlot();
+  Status PerformSlotRangeMigration(const std::string &node_id, std::string &dst_ip, int dst_port,
+                                   const SlotRange &range, SyncMigrateContext *blocking_ctx = nullptr);
+  void ReleaseForbiddenSlotRange();
   void SetMaxMigrationSpeed(int value) {
     if (value >= 0) max_migration_speed_ = value;
   }
@@ -101,8 +109,8 @@ class SlotMigrator : public redis::Database {
   void SetStopMigrationFlag(bool value) { stop_migration_ = value; }
   bool IsMigrationInProgress() const { return migration_state_ == MigrationState::kStarted; }
   SlotMigrationStage GetCurrentSlotMigrationStage() const { return current_stage_; }
-  int16_t GetForbiddenSlot() const { return forbidden_slot_; }
-  int16_t GetMigratingSlot() const { return migrating_slot_; }
+  SlotRange GetForbiddenSlotRange() const { return forbidden_slot_range_; }
+  SlotRange GetMigratingSlotRange() const { return slot_range_; }
   std::string GetDstNode() const { return dst_node_; }
   void GetMigrationInfo(std::string *info) const;
   void CancelSyncCtx();
@@ -110,7 +118,7 @@ class SlotMigrator : public redis::Database {
  private:
   void loop();
   void runMigrationProcess();
-  bool isTerminated() { return thread_state_ == ThreadState::Terminated; }
+  bool isTerminated() const { return thread_state_ == ThreadState::Terminated; }
   Status startMigration();
   Status sendSnapshot();
   Status syncWAL();
@@ -149,7 +157,7 @@ class SlotMigrator : public redis::Database {
   bool catchUpIncrementalWAL();
   Status migrateIncrementalDataByRawKV(uint64_t end_seq, BatchSender *batch_sender);
 
-  void setForbiddenSlot(int16_t slot);
+  void setForbiddenSlotRange(const SlotRange &slot_range);
   std::unique_lock<std::mutex> blockingLock() { return std::unique_lock<std::mutex>(blocking_mutex_); }
 
   void resumeSyncCtx(const Status &migrate_result);
@@ -157,11 +165,11 @@ class SlotMigrator : public redis::Database {
   enum class ParserState { ArrayLen, BulkLen, BulkData, ArrayData, OneRspEnd };
   enum class ThreadState { Uninitialized, Running, Terminated };
 
-  static const int kDefaultMaxPipelineSize = 16;
-  static const int kDefaultMaxMigrationSpeed = 4096;
-  static const int kDefaultSequenceGapLimit = 10000;
-  static const int kMaxItemsInCommand = 16;  // number of items in every write command of complex keys
-  static const int kMaxLoopTimes = 10;
+  static constexpr int kDefaultMaxPipelineSize = 16;
+  static constexpr int kDefaultMaxMigrationSpeed = 4096;
+  static constexpr int kDefaultSequenceGapLimit = 10000;
+  static constexpr int kMaxItemsInCommand = 16;  // number of items in every write command of complex keys
+  static constexpr int kMaxLoopTimes = 10;
 
   Server *srv_;
 
@@ -182,7 +190,7 @@ class SlotMigrator : public redis::Database {
   std::thread t_;
   std::mutex job_mutex_;
   std::condition_variable job_cv_;
-  std::unique_ptr<SlotMigrationJob> migration_job_;
+  std::unique_ptr<SlotMigrationJob> migration_job_;  // GUARDED_BY(job_mutex_)
 
   std::string dst_node_;
   std::string dst_ip_;
@@ -190,9 +198,12 @@ class SlotMigrator : public redis::Database {
   UniqueFD dst_fd_;
 
   MigrationType migration_type_ = MigrationType::kRedisCommand;
-  std::atomic<int16_t> forbidden_slot_ = -1;
-  std::atomic<int16_t> migrating_slot_ = -1;
-  int16_t migrate_failed_slot_ = -1;
+
+  static_assert(std::atomic<SlotRange>::is_always_lock_free, "SlotRange is not lock free.");
+  std::atomic<SlotRange> forbidden_slot_range_ = SlotRange{-1, -1};
+  std::atomic<SlotRange> slot_range_ = SlotRange{-1, -1};
+  std::atomic<SlotRange> migrate_failed_slot_range_ = SlotRange{-1, -1};
+
   std::atomic<bool> stop_migration_ = false;  // if is true migration will be stopped but the thread won't be destroyed
   const rocksdb::Snapshot *slot_snapshot_ = nullptr;
   uint64_t wal_begin_seq_ = 0;
