@@ -28,6 +28,7 @@
 #include "error_constants.h"
 #include "event_util.h"
 #include "server/server.h"
+#include "status.h"
 #include "time_util.h"
 #include "types/redis_stream.h"
 
@@ -51,7 +52,8 @@ class CommandXAck : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     uint64_t acknowledged = 0;
-    auto s = stream_db.DeletePelEntries(stream_name_, group_name_, entry_ids_, &acknowledged);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.DeletePelEntries(ctx, stream_name_, group_name_, entry_ids_, &acknowledged);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -178,7 +180,8 @@ class CommandXAdd : public Commander {
 
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     StreamEntryID entry_id;
-    auto s = stream_db.Add(stream_name_, options, name_value_pairs_, &entry_id);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.Add(ctx, stream_name_, options, name_value_pairs_, &entry_id);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -224,7 +227,8 @@ class CommandXDel : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     uint64_t deleted = 0;
-    auto s = stream_db.DeleteEntries(args_[1], ids_, &deleted);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.DeleteEntries(ctx, args_[1], ids_, &deleted);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -318,16 +322,16 @@ class CommandXClaim : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     StreamClaimResult result;
-    auto s = stream_db.ClaimPelEntries(stream_name_, group_name_, consumer_name_, min_idle_time_ms_, entry_ids_,
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.ClaimPelEntries(ctx, stream_name_, group_name_, consumer_name_, min_idle_time_ms_, entry_ids_,
                                        stream_claim_options_, &result);
     if (!s.ok()) {
+      if (s.IsNotFound()) {
+        return {Status::RedisNoGroup, "No such key '" + stream_name_ + "' or consumer group '" + group_name_ + "'"};
+      }
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    if (s.IsNotFound()) {
-      return {Status::RedisExecErr, errNoSuchKey};
-    }
-
     if (!stream_claim_options_.just_id) {
       output->append(redis::MultiLen(result.entries.size()));
 
@@ -408,11 +412,11 @@ class CommandAutoClaim : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     StreamAutoClaimResult result;
-    auto s = stream_db.AutoClaim(key_name_, group_name_, consumer_name_, options_, &result);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.AutoClaim(ctx, key_name_, group_name_, consumer_name_, options_, &result);
     if (!s.ok()) {
       if (s.IsNotFound()) {
-        return {Status::RedisExecErr,
-                "NOGROUP No such key '" + key_name_ + "' or consumer group '" + group_name_ + "'"};
+        return {Status::RedisNoGroup, "No such key '" + key_name_ + "' or consumer group '" + group_name_ + "'"};
       }
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -420,7 +424,8 @@ class CommandAutoClaim : public Commander {
   }
 
  private:
-  Status sendResults(Connection *conn, const StreamAutoClaimResult &result, std::string *output) const {
+  Status sendResults([[maybe_unused]] Connection *conn, const StreamAutoClaimResult &result,
+                     std::string *output) const {
     output->append(redis::MultiLen(3));
     output->append(redis::BulkString(result.next_claim_id));
     output->append(redis::MultiLen(result.entries.size()));
@@ -533,9 +538,14 @@ class CommandXGroup : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
+    engine::Context ctx(srv->storage);
+
     if (subcommand_ == "create") {
-      auto s = stream_db.CreateGroup(stream_name_, xgroup_create_options_, group_name_);
+      auto s = stream_db.CreateGroup(ctx, stream_name_, xgroup_create_options_, group_name_);
       if (!s.ok()) {
+        if (s.IsBusy()) {
+          return {Status::RedisBusyGroup, "consumer group name '" + group_name_ + "' already exists"};
+        }
         return {Status::RedisExecErr, s.ToString()};
       }
 
@@ -544,7 +554,7 @@ class CommandXGroup : public Commander {
 
     if (subcommand_ == "destroy") {
       uint64_t delete_cnt = 0;
-      auto s = stream_db.DestroyGroup(stream_name_, group_name_, &delete_cnt);
+      auto s = stream_db.DestroyGroup(ctx, stream_name_, group_name_, &delete_cnt);
       if (!s.ok()) {
         return {Status::RedisExecErr, s.ToString()};
       }
@@ -558,8 +568,11 @@ class CommandXGroup : public Commander {
 
     if (subcommand_ == "createconsumer") {
       int created_number = 0;
-      auto s = stream_db.CreateConsumer(stream_name_, group_name_, consumer_name_, &created_number);
+      auto s = stream_db.CreateConsumer(ctx, stream_name_, group_name_, consumer_name_, &created_number);
       if (!s.ok()) {
+        if (s.IsNotFound()) {
+          return {Status::RedisNoGroup, "No such consumer group " + group_name_ + " for key name " + stream_name_};
+        }
         return {Status::RedisExecErr, s.ToString()};
       }
 
@@ -568,8 +581,11 @@ class CommandXGroup : public Commander {
 
     if (subcommand_ == "delconsumer") {
       uint64_t deleted_pel = 0;
-      auto s = stream_db.DestroyConsumer(stream_name_, group_name_, consumer_name_, deleted_pel);
+      auto s = stream_db.DestroyConsumer(ctx, stream_name_, group_name_, consumer_name_, deleted_pel);
       if (!s.ok()) {
+        if (s.IsNotFound()) {
+          return {Status::RedisNoGroup, "No such consumer group " + group_name_ + " for key name " + stream_name_};
+        }
         return {Status::RedisExecErr, s.ToString()};
       }
 
@@ -577,8 +593,11 @@ class CommandXGroup : public Commander {
     }
 
     if (subcommand_ == "setid") {
-      auto s = stream_db.GroupSetId(stream_name_, group_name_, xgroup_create_options_);
+      auto s = stream_db.GroupSetId(ctx, stream_name_, group_name_, xgroup_create_options_);
       if (!s.ok()) {
+        if (s.IsNotFound()) {
+          return {Status::RedisNoGroup, "No such consumer group " + group_name_ + " for key name " + stream_name_};
+        }
         return {Status::RedisExecErr, s.ToString()};
       }
 
@@ -622,7 +641,8 @@ class CommandXLen : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     uint64_t len = 0;
-    auto s = stream_db.Len(args_[1], options_, &len);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.Len(ctx, args_[1], options_, &len);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -655,10 +675,10 @@ class CommandXInfo : public Commander {
 
         count_ = *parse_result;
       }
-      // } else if (val == "groups" && args.size() == 3) {
-      //   subcommand_ = "groups";
-      // } else if (val == "consumers" && args.size() == 4) {
-      //   subcommand_ = "consumers";
+    } else if (val == "groups" && args.size() == 3) {
+      subcommand_ = "groups";
+    } else if (val == "consumers" && args.size() == 4) {
+      subcommand_ = "consumers";
     } else {
       return {Status::RedisParseErr, errUnknownSubcommandOrWrongArguments};
     }
@@ -689,7 +709,8 @@ class CommandXInfo : public Commander {
   Status getStreamInfo(Server *srv, Connection *conn, std::string *output) {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     redis::StreamInfo info;
-    auto s = stream_db.GetStreamInfo(args_[2], full_, count_, &info);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.GetStreamInfo(ctx, args_[2], full_, count_, &info);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -746,7 +767,8 @@ class CommandXInfo : public Commander {
   Status getGroupInfo(Server *srv, Connection *conn, std::string *output) {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     std::vector<std::pair<std::string, StreamConsumerGroupMetadata>> result_vector;
-    auto s = stream_db.GetGroupInfo(args_[2], result_vector);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.GetGroupInfo(ctx, args_[2], result_vector);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -786,7 +808,8 @@ class CommandXInfo : public Commander {
   Status getConsumerInfo(Server *srv, Connection *conn, std::string *output) {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
     std::vector<std::pair<std::string, redis::StreamConsumerMetadata>> result_vector;
-    auto s = stream_db.GetConsumerInfo(args_[2], args_[3], result_vector);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.GetConsumerInfo(ctx, args_[2], args_[3], result_vector);
 
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -814,6 +837,100 @@ class CommandXInfo : public Commander {
   }
 };
 
+class CommandXPending : public Commander {
+ public:
+  Status Parse(const std::vector<std::string> &args) override {
+    CommandParser parser(args, 1);
+    stream_name_ = GET_OR_RET(parser.TakeStr());
+    group_name_ = GET_OR_RET(parser.TakeStr());
+    if (parser.EatEqICase("idle")) {
+      options_.idle_time = GET_OR_RET(parser.TakeInt<uint64_t>());
+      options_.with_time = true;
+    }
+
+    if (parser.Good()) {
+      std::string start_id, end_id;
+      start_id = GET_OR_RET(parser.TakeStr());
+      end_id = GET_OR_RET(parser.TakeStr());
+      if (start_id != "-") {
+        auto s = ParseStreamEntryID(start_id, &options_.start_id);
+        if (!s.IsOK()) {
+          return s;
+        }
+      }
+
+      if (end_id != "+") {
+        auto s = ParseStreamEntryID(start_id, &options_.end_id);
+        if (!s.IsOK()) {
+          return s;
+        }
+      }
+
+      options_.count = GET_OR_RET(parser.TakeInt<uint64_t>());
+      options_.with_count = true;
+      if (parser.Good()) {
+        options_.consumer = GET_OR_RET(parser.TakeStr());
+        options_.with_consumer = true;
+      }
+    }
+    return Status::OK();
+  }
+
+  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+    redis::Stream stream_db(srv->storage, conn->GetNamespace());
+    std::vector<std::pair<std::string, int>> pending_infos;
+    StreamGetPendingEntryResult results;
+    options_.stream_name = stream_name_;
+    options_.group_name = group_name_;
+    std::vector<StreamNACK> ext_results;
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.GetPendingEntries(ctx, options_, results, ext_results);
+    if (!s.ok()) {
+      return {Status::RedisExecErr, s.ToString()};
+    }
+    if (options_.with_count) {
+      return SendExtResults(conn, output, ext_results);
+    }
+    return SendResults(conn, output, results);
+  }
+
+  static Status SendResults([[maybe_unused]] Connection *conn, std::string *output,
+                            StreamGetPendingEntryResult &results) {
+    output->append(redis::MultiLen(3 + results.consumer_infos.size()));
+    output->append(redis::Integer(results.pending_number));
+    output->append(redis::BulkString(results.first_entry_id.ToString()));
+    output->append(redis::BulkString(results.last_entry_id.ToString()));
+    output->append(redis::MultiLen(results.consumer_infos.size()));
+    for (const auto &entry : results.consumer_infos) {
+      output->append(redis::MultiLen(2));
+      output->append(redis::BulkString(entry.first));
+      output->append(redis::BulkString(std::to_string(entry.second)));
+    }
+
+    return Status::OK();
+  }
+
+  static Status SendExtResults([[maybe_unused]] Connection *conn, std::string *output,
+                               std::vector<StreamNACK> &ext_results) {
+    output->append(redis::MultiLen(ext_results.size()));
+    for (const auto &entry : ext_results) {
+      output->append(redis::MultiLen(4));
+      output->append(redis::BulkString(entry.id.ToString()));
+      output->append(redis::BulkString(entry.pel_entry.consumer_name));
+      output->append(redis::Integer(entry.pel_entry.last_delivery_time_ms));
+      output->append(redis::Integer(entry.pel_entry.last_delivery_count));
+    }
+
+    return Status::OK();
+  }
+
+ private:
+  std::string group_name_;
+  std::string stream_name_;
+  std::string consumer_name_;
+  StreamPendingOptions options_;
+};
+
 class CommandXRange : public Commander {
  public:
   Status Parse(const std::vector<std::string> &args) override {
@@ -881,7 +998,8 @@ class CommandXRange : public Commander {
     options.exclude_end = exclude_end_;
 
     std::vector<StreamEntry> result;
-    auto s = stream_db.Range(stream_name_, options, &result);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.Range(ctx, stream_name_, options, &result);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -974,7 +1092,8 @@ class CommandXRevRange : public Commander {
     options.exclude_end = exclude_end_;
 
     std::vector<StreamEntry> result;
-    auto s = stream_db.Range(stream_name_, options, &result);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.Range(ctx, stream_name_, options, &result);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -1088,6 +1207,7 @@ class CommandXRead : public Commander,
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
     std::vector<redis::StreamReadResult> results;
+    engine::Context ctx(srv->storage);
 
     for (size_t i = 0; i < streams_.size(); ++i) {
       if (latest_marks_[i]) {
@@ -1097,14 +1217,14 @@ class CommandXRead : public Commander,
       redis::StreamRangeOptions options;
       options.reverse = false;
       options.start = ids_[i];
-      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.end = StreamEntryID::Maximum();
       options.with_count = with_count_;
       options.count = count_;
       options.exclude_start = true;
       options.exclude_end = false;
 
       std::vector<StreamEntry> result;
-      auto s = stream_db.Range(streams_[i], options, &result);
+      auto s = stream_db.Range(ctx, streams_[i], options, &result);
       if (!s.ok() && !s.IsNotFound()) {
         return {Status::RedisExecErr, s.ToString()};
       }
@@ -1120,7 +1240,7 @@ class CommandXRead : public Commander,
         return Status::OK();  // No blocking in multi-exec
       }
 
-      return BlockingRead(srv, conn, &stream_db);
+      return BlockingRead(ctx, srv, conn, &stream_db);
     }
 
     if (!block_ && results.empty()) {
@@ -1148,7 +1268,8 @@ class CommandXRead : public Commander,
     return Status::OK();
   }
 
-  Status BlockingRead(Server *srv, Connection *conn, redis::Stream *stream_db) {
+  Status BlockingRead([[maybe_unused]] const engine::Context &ctx, Server *srv, Connection *conn,
+                      redis::Stream *stream_db) {
     if (!with_count_) {
       with_count_ = true;
       count_ = blocked_default_count_;
@@ -1157,7 +1278,8 @@ class CommandXRead : public Commander,
     for (size_t i = 0; i < streams_.size(); ++i) {
       if (latest_marks_[i]) {
         StreamEntryID last_generated_id;
-        auto s = stream_db->GetLastGeneratedID(streams_[i], &last_generated_id);
+        engine::Context ctx(srv->storage);
+        auto s = stream_db->GetLastGeneratedID(ctx, streams_[i], &last_generated_id);
         if (!s.ok()) {
           return {Status::RedisExecErr, s.ToString()};
         }
@@ -1203,19 +1325,19 @@ class CommandXRead : public Commander,
     redis::Stream stream_db(srv_->storage, conn_->GetNamespace());
 
     std::vector<StreamReadResult> results;
-
+    engine::Context ctx(srv_->storage);
     for (size_t i = 0; i < streams_.size(); ++i) {
       redis::StreamRangeOptions options;
       options.reverse = false;
       options.start = ids_[i];
-      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.end = StreamEntryID::Maximum();
       options.with_count = with_count_;
       options.count = count_;
       options.exclude_start = true;
       options.exclude_end = false;
 
       std::vector<StreamEntry> result;
-      auto s = stream_db.Range(streams_[i], options, &result);
+      auto s = stream_db.Range(ctx, streams_[i], options, &result);
       if (!s.ok() && !s.IsNotFound()) {
         conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
         return;
@@ -1262,7 +1384,7 @@ class CommandXRead : public Commander,
     conn_->OnEvent(bev, events);
   }
 
-  void TimerCB(int, int16_t events) {
+  void TimerCB(int, [[maybe_unused]] int16_t events) {
     conn_->Reply(conn_->NilString());
 
     timer_.reset();
@@ -1388,21 +1510,21 @@ class CommandXReadGroup : public Commander,
 
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
-
+    engine::Context ctx(srv->storage);
     std::vector<redis::StreamReadResult> results;
 
     for (size_t i = 0; i < streams_.size(); ++i) {
       redis::StreamRangeOptions options;
       options.reverse = false;
       options.start = ids_[i];
-      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.end = StreamEntryID::Maximum();
       options.with_count = with_count_;
       options.count = count_;
       options.exclude_start = true;
       options.exclude_end = false;
 
       std::vector<StreamEntry> result;
-      auto s = stream_db.RangeWithPending(streams_[i], options, &result, group_name_, consumer_name_, noack_,
+      auto s = stream_db.RangeWithPending(ctx, streams_[i], options, &result, group_name_, consumer_name_, noack_,
                                           latest_marks_[i]);
       if (!s.ok() && !s.IsNotFound()) {
         return {Status::RedisExecErr, s.ToString()};
@@ -1452,7 +1574,7 @@ class CommandXReadGroup : public Commander,
     return Status::OK();
   }
 
-  Status BlockingRead(Server *srv, Connection *conn, redis::Stream *stream_db) {
+  Status BlockingRead(Server *srv, Connection *conn, [[maybe_unused]] redis::Stream *stream_db) {
     if (!with_count_) {
       with_count_ = true;
       count_ = blocked_default_count_;
@@ -1495,19 +1617,19 @@ class CommandXReadGroup : public Commander,
     redis::Stream stream_db(srv_->storage, conn_->GetNamespace());
 
     std::vector<StreamReadResult> results;
-
+    engine::Context ctx(srv_->storage);
     for (size_t i = 0; i < streams_.size(); ++i) {
       redis::StreamRangeOptions options;
       options.reverse = false;
       options.start = ids_[i];
-      options.end = StreamEntryID{UINT64_MAX, UINT64_MAX};
+      options.end = StreamEntryID::Maximum();
       options.with_count = with_count_;
       options.count = count_;
       options.exclude_start = true;
       options.exclude_end = false;
 
       std::vector<StreamEntry> result;
-      auto s = stream_db.RangeWithPending(streams_[i], options, &result, group_name_, consumer_name_, noack_,
+      auto s = stream_db.RangeWithPending(ctx, streams_[i], options, &result, group_name_, consumer_name_, noack_,
                                           latest_marks_[i]);
       if (!s.ok() && !s.IsNotFound()) {
         conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
@@ -1555,7 +1677,7 @@ class CommandXReadGroup : public Commander,
     conn_->OnEvent(bev, events);
   }
 
-  void TimerCB(int, int16_t events) {
+  void TimerCB(int, [[maybe_unused]] int16_t events) {
     conn_->Reply(conn_->NilString());
 
     timer_.reset();
@@ -1664,7 +1786,8 @@ class CommandXTrim : public Commander {
     options.min_id = min_id_;
 
     uint64_t removed = 0;
-    auto s = stream_db.Trim(args_[1], options, &removed);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.Trim(ctx, args_[1], options, &removed);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -1719,7 +1842,8 @@ class CommandXSetId : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::Stream stream_db(srv->storage, conn->GetNamespace());
 
-    auto s = stream_db.SetId(stream_name_, last_id_, entries_added_, max_deleted_id_);
+    engine::Context ctx(srv->storage);
+    auto s = stream_db.SetId(ctx, stream_name_, last_id_, entries_added_, max_deleted_id_);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -1736,20 +1860,20 @@ class CommandXSetId : public Commander {
   std::optional<uint64_t> entries_added_;
 };
 
-REDIS_REGISTER_COMMANDS(
-    // MakeCmdAttr<CommandXAck>("xack", -4, "write no-dbsize-check", 1, 1, 1),
-    MakeCmdAttr<CommandXAdd>("xadd", -5, "write", 1, 1, 1),
-    MakeCmdAttr<CommandXDel>("xdel", -3, "write no-dbsize-check", 1, 1, 1),
-    // MakeCmdAttr<CommandXClaim>("xclaim", -6, "write", 1, 1, 1),
-    // MakeCmdAttr<CommandAutoClaim>("xautoclaim", -6, "write", 1, 1, 1),
-    // MakeCmdAttr<CommandXGroup>("xgroup", -4, "write", 2, 2, 1),
-    MakeCmdAttr<CommandXLen>("xlen", -2, "read-only", 1, 1, 1),
-    MakeCmdAttr<CommandXInfo>("xinfo", -2, "read-only", 0, 0, 0),
-    MakeCmdAttr<CommandXRange>("xrange", -4, "read-only", 1, 1, 1),
-    MakeCmdAttr<CommandXRevRange>("xrevrange", -2, "read-only", 1, 1, 1),
-    MakeCmdAttr<CommandXRead>("xread", -4, "read-only", 0, 0, 0),
-    // MakeCmdAttr<CommandXReadGroup>("xreadgroup", -7, "write", 0, 0, 0),
-    MakeCmdAttr<CommandXTrim>("xtrim", -4, "write no-dbsize-check", 1, 1, 1),
-    MakeCmdAttr<CommandXSetId>("xsetid", -3, "write", 1, 1, 1))
+REDIS_REGISTER_COMMANDS(Stream, MakeCmdAttr<CommandXAck>("xack", -4, "write no-dbsize-check", 1, 1, 1),
+                        MakeCmdAttr<CommandXAdd>("xadd", -5, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandXDel>("xdel", -3, "write no-dbsize-check", 1, 1, 1),
+                        MakeCmdAttr<CommandXClaim>("xclaim", -6, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandAutoClaim>("xautoclaim", -6, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandXGroup>("xgroup", -4, "write", 2, 2, 1),
+                        MakeCmdAttr<CommandXLen>("xlen", -2, "read-only", 1, 1, 1),
+                        MakeCmdAttr<CommandXInfo>("xinfo", -2, "read-only", 0, 0, 0),
+                        MakeCmdAttr<CommandXPending>("xpending", -3, "read-only", 1, 1, 1),
+                        MakeCmdAttr<CommandXRange>("xrange", -4, "read-only", 1, 1, 1),
+                        MakeCmdAttr<CommandXRevRange>("xrevrange", -2, "read-only", 1, 1, 1),
+                        MakeCmdAttr<CommandXRead>("xread", -4, "read-only", 0, 0, 0),
+                        MakeCmdAttr<CommandXReadGroup>("xreadgroup", -7, "write", 0, 0, 0),
+                        MakeCmdAttr<CommandXTrim>("xtrim", -4, "write no-dbsize-check", 1, 1, 1),
+                        MakeCmdAttr<CommandXSetId>("xsetid", -3, "write", 1, 1, 1))
 
 }  // namespace redis
