@@ -25,6 +25,7 @@
 #include <strings.h>
 
 #include <algorithm>
+#include <cstdint>
 #include <cstring>
 #include <fstream>
 #include <iostream>
@@ -184,12 +185,15 @@ Config::Config() {
       {"unixsocketperm", true, new OctalField(&unixsocketperm, 0777, 1, INT_MAX)},
       {"log-retention-days", false, new IntField(&log_retention_days, -1, -1, INT_MAX)},
       {"persist-cluster-nodes-enabled", false, new YesNoField(&persist_cluster_nodes_enabled, true)},
-      {"redis-cursor-compatible", false, new YesNoField(&redis_cursor_compatible, false)},
+      {"redis-cursor-compatible", false, new YesNoField(&redis_cursor_compatible, true)},
       {"resp3-enabled", false, new YesNoField(&resp3_enabled, false)},
       {"repl-namespace-enabled", false, new YesNoField(&repl_namespace_enabled, false)},
+      {"proto-max-bulk-len", false,
+       new IntWithUnitField<uint64_t>(&proto_max_bulk_len, std::to_string(512 * MiB), 1 * MiB, UINT64_MAX)},
       {"json-max-nesting-depth", false, new IntField(&json_max_nesting_depth, 1024, 0, INT_MAX)},
       {"json-storage-format", false,
        new EnumField<JsonStorageFormat>(&json_storage_format, json_storage_formats, JsonStorageFormat::JSON)},
+      {"txn-context-enabled", true, new YesNoField(&txn_context_enabled, false)},
 
       /* rocksdb options */
       {"rocksdb.compression", false,
@@ -203,7 +207,7 @@ Config::Config() {
       {"rocksdb.target_file_size_base", false, new IntField(&rocks_db.target_file_size_base, 128, 1, 1024)},
       {"rocksdb.max_background_compactions", false, new IntField(&rocks_db.max_background_compactions, 2, -1, 32)},
       {"rocksdb.max_background_flushes", true, new IntField(&rocks_db.max_background_flushes, 2, -1, 32)},
-      {"rocksdb.max_sub_compactions", false, new IntField(&rocks_db.max_sub_compactions, 2, 0, 16)},
+      {"rocksdb.max_subcompactions", false, new IntField(&rocks_db.max_subcompactions, 2, 0, 16)},
       {"rocksdb.delayed_write_rate", false, new Int64Field(&rocks_db.delayed_write_rate, 0, 0, INT64_MAX)},
       {"rocksdb.wal_ttl_seconds", true, new IntField(&rocks_db.wal_ttl_seconds, 3 * 3600, 0, INT_MAX)},
       {"rocksdb.wal_size_limit_mb", true, new IntField(&rocks_db.wal_size_limit_mb, 16384, 0, INT_MAX)},
@@ -250,6 +254,8 @@ Config::Config() {
       {"rocksdb.write_options.low_pri", true, new YesNoField(&rocks_db.write_options.low_pri, false)},
       {"rocksdb.write_options.memtable_insert_hint_per_batch", true,
        new YesNoField(&rocks_db.write_options.memtable_insert_hint_per_batch, false)},
+      {"rocksdb.write_options.write_batch_max_bytes", false,
+       new IntField(&rocks_db.write_options.write_batch_max_bytes, 0, 0, INT_MAX)},
 
       /* rocksdb read options */
       {"rocksdb.read_options.async_io", false, new YesNoField(&rocks_db.read_options.async_io, true)},
@@ -268,7 +274,7 @@ Config::Config() {
 void Config::initFieldValidator() {
   std::map<std::string, ValidateFn> validators = {
       {"requirepass",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          if (v.empty() && !load_tokens.empty()) {
            return {Status::NotOK, "requirepass empty not allowed while the namespace exists"};
          }
@@ -278,29 +284,29 @@ void Config::initFieldValidator() {
          return Status::OK();
        }},
       {"masterauth",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          if (load_tokens.find(v) != load_tokens.end()) {
            return {Status::NotOK, "masterauth is duplicated with namespace tokens"};
          }
          return Status::OK();
        }},
       {"compact-cron",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          std::vector<std::string> args = util::Split(v, " \t");
          return compact_cron.SetScheduleTime(args);
        }},
       {"bgsave-cron",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          std::vector<std::string> args = util::Split(v, " \t");
          return bgsave_cron.SetScheduleTime(args);
        }},
       {"dbsize-scan-cron",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          std::vector<std::string> args = util::Split(v, " \t");
          return dbsize_scan_cron.SetScheduleTime(args);
        }},
       {"compaction-checker-range",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          if (!compaction_checker_cron_str_.empty()) {
            return {Status::NotOK, "compaction-checker-range cannot be set while compaction-checker-cron is set"};
          }
@@ -311,12 +317,12 @@ void Config::initFieldValidator() {
          return compaction_checker_cron.SetScheduleTime({"*", v, "*", "*", "*"});
        }},
       {"compaction-checker-cron",
-       [this](const std::string &k, const std::string &v) -> Status {
+       [this]([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          std::vector<std::string> args = util::Split(v, " \t");
          return compaction_checker_cron.SetScheduleTime(args);
        }},
       {"rename-command",
-       [](const std::string &k, const std::string &v) -> Status {
+       []([[maybe_unused]] const std::string &k, const std::string &v) -> Status {
          std::vector<std::string> all_args = util::Split(v, "\n");
          for (auto &p : all_args) {
            std::vector<std::string> args = util::Split(p, " \t");
@@ -360,7 +366,8 @@ void Config::initFieldCallback() {
     if (!srv) return Status::OK();  // srv is nullptr when load config from file
     return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k), v);
   };
-  auto set_compression_type_cb = [](Server *srv, const std::string &k, const std::string &v) -> Status {
+  auto set_compression_type_cb = [](Server *srv, [[maybe_unused]] const std::string &k,
+                                    const std::string &v) -> Status {
     if (!srv) return Status::OK();
 
     std::string compression_option;
@@ -385,7 +392,7 @@ void Config::initFieldCallback() {
     return srv->storage->SetOptionForAllColumnFamilies("compression_per_level", compression_levels);
   };
 #ifdef ENABLE_OPENSSL
-  auto set_tls_option = [](Server *srv, const std::string &k, const std::string &v) {
+  auto set_tls_option = [](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) {
     if (!srv) return Status::OK();  // srv is nullptr when load config from file
     auto new_ctx = CreateSSLContext(srv->GetConfig());
     if (!new_ctx) {
@@ -399,13 +406,14 @@ void Config::initFieldCallback() {
   std::map<std::string, CallbackFn> callbacks =
       {
           {"workers",
-           [](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->AdjustWorkerThreads();
              return Status::OK();
            }},
           {"dir",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this]([[maybe_unused]] Server *srv, [[maybe_unused]] const std::string &k,
+                  [[maybe_unused]] const std::string &v) -> Status {
              db_dir = dir + "/db";
              if (log_dir.empty()) log_dir = dir;
              checkpoint_dir = dir + "/checkpoint";
@@ -416,7 +424,7 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"backup-dir",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, const std::string &v) -> Status {
              std::string previous_backup;
              {
                // Note: currently, backup_mu_ may block by backing up or purging,
@@ -432,24 +440,25 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"cluster-enabled",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this]([[maybe_unused]] Server *srv, [[maybe_unused]] const std::string &k,
+                  [[maybe_unused]] const std::string &v) -> Status {
              if (cluster_enabled) slot_id_encoded = true;
              return Status::OK();
            }},
           {"bind",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this]([[maybe_unused]] Server *srv, [[maybe_unused]] const std::string &k, const std::string &v) -> Status {
              std::vector<std::string> args = util::Split(v, " \t");
              binds = std::move(args);
              return Status::OK();
            }},
           {"maxclients",
-           [](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->AdjustOpenFilesLimit();
              return Status::OK();
            }},
           {"slaveof",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this]([[maybe_unused]] Server *srv, [[maybe_unused]] const std::string &k, const std::string &v) -> Status {
              if (v.empty()) {
                return Status::OK();
              }
@@ -466,7 +475,7 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"profiling-sample-commands",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this]([[maybe_unused]] Server *srv, [[maybe_unused]] const std::string &k, const std::string &v) -> Status {
              std::vector<std::string> cmds = util::Split(v, ",");
              profiling_sample_all_commands = false;
              profiling_sample_commands.clear();
@@ -485,67 +494,67 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"slowlog-max-len",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->GetSlowLog()->SetMaxEntries(slowlog_max_len);
              return Status::OK();
            }},
           {"max-db-size",
-           [](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->storage->CheckDBSizeLimit();
              return Status::OK();
            }},
           {"max-io-mb",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->storage->SetIORateLimit(max_io_mb);
              return Status::OK();
            }},
           {"profiling-sample-record-max-len",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->GetPerfLog()->SetMaxEntries(profiling_sample_record_max_len);
              return Status::OK();
            }},
           {"migrate-speed",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (cluster_enabled) srv->slot_migrator->SetMaxMigrationSpeed(migrate_speed);
              return Status::OK();
            }},
           {"migrate-pipeline-size",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (cluster_enabled) srv->slot_migrator->SetMaxPipelineSize(pipeline_size);
              return Status::OK();
            }},
           {"migrate-sequence-gap",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (cluster_enabled) srv->slot_migrator->SetSequenceGapLimit(sequence_gap);
              return Status::OK();
            }},
           {"migrate-batch-rate-limit-mb",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->slot_migrator->SetMigrateBatchRateLimit(migrate_batch_rate_limit_mb * MiB);
              return Status::OK();
            }},
           {"migrate-batch-size-kb",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              srv->slot_migrator->SetMigrateBatchSize(migrate_batch_size_kb * KiB);
              return Status::OK();
            }},
           {"log-level",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              FLAGS_minloglevel = log_level;
              return Status::OK();
            }},
           {"log-retention-days",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (util::ToLower(log_dir) == "stdout") {
                return {Status::NotOK, "can't set the 'log-retention-days' when the log dir is stdout"};
@@ -559,7 +568,7 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"persist-cluster-nodes-enabled",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, [[maybe_unused]] const std::string &k, const std::string &v) -> Status {
              if (!srv || !cluster_enabled) return Status::OK();
              auto nodes_file_path = NodesFilePath();
              if (v == "yes") {
@@ -570,19 +579,19 @@ void Config::initFieldCallback() {
              return Status::OK();
            }},
           {"repl-namespace-enabled",
-           [](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [](Server *srv, [[maybe_unused]] const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              return srv->GetNamespace()->LoadAndRewrite();
            }},
 
           {"rocksdb.target_file_size_base",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k),
                                                                 std::to_string(rocks_db.target_file_size_base * MiB));
            }},
           {"rocksdb.write_buffer_size",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k),
                                                                 std::to_string(rocks_db.write_buffer_size * MiB));
@@ -594,12 +603,12 @@ void Config::initFieldCallback() {
              return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k), disable_auto_compactions);
            }},
           {"rocksdb.max_total_wal_size",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              return srv->storage->SetDBOption(TrimRocksDbPrefix(k), std::to_string(rocks_db.max_total_wal_size * MiB));
            }},
           {"rocksdb.enable_blob_files",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              std::string enable_blob_files = rocks_db.enable_blob_files ? "true" : "false";
              return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k), enable_blob_files);
@@ -613,7 +622,7 @@ void Config::initFieldCallback() {
              return srv->storage->SetOptionForAllColumnFamilies(TrimRocksDbPrefix(k), v);
            }},
           {"rocksdb.blob_file_size",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (!rocks_db.enable_blob_files) {
                return {Status::NotOK, errBlobDbNotEnabled};
@@ -656,7 +665,7 @@ void Config::initFieldCallback() {
              return srv->storage->SetDBOption(TrimRocksDbPrefix(k), level_compaction_dynamic_level_bytes);
            }},
           {"rocksdb.max_bytes_for_level_base",
-           [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+           [this](Server *srv, const std::string &k, [[maybe_unused]] const std::string &v) -> Status {
              if (!srv) return Status::OK();
              if (!rocks_db.level_compaction_dynamic_level_bytes) {
                return {Status::NotOK, errLevelCompactionDynamicLevelBytesNotSet};
@@ -677,6 +686,7 @@ void Config::initFieldCallback() {
           {"rocksdb.delayed_write_rate", set_db_option_cb},
           {"rocksdb.max_background_compactions", set_db_option_cb},
           {"rocksdb.max_background_flushes", set_db_option_cb},
+          {"rocksdb.max_subcompactions", set_db_option_cb},
           {"rocksdb.compaction_readahead_size", set_db_option_cb},
           {"rocksdb.max_background_jobs", set_db_option_cb},
 
@@ -884,7 +894,7 @@ Status Config::Set(Server *srv, std::string key, const std::string &value) {
     if (!s.IsOK()) return s.Prefixed("invalid value");
   }
 
-  auto origin_value = field->ToString();
+  auto origin_value = field->ToStringForRewrite();
   auto s = field->Set(value);
   if (!s.IsOK()) return s.Prefixed("failed to set new value");
 
@@ -921,7 +931,7 @@ Status Config::Rewrite(const std::map<std::string, std::string> &tokens) {
       // so skip it here to avoid rewriting it as new item.
       continue;
     }
-    new_config[iter.first] = iter.second->ToString();
+    new_config[iter.first] = iter.second->ToStringForRewrite();
   }
 
   std::string namespace_prefix = "namespace.";
