@@ -33,6 +33,7 @@ enum class IndexOnDataType : uint8_t {
 };
 
 inline constexpr auto kErrorInsufficientLength = "insufficient length while decoding metadata";
+inline constexpr auto kErrorIncorrectLength = "length is too short or too long to be parsed as a vector";
 
 class IndexMetadata {
  public:
@@ -76,6 +77,23 @@ enum class IndexFieldType : uint8_t {
   TAG = 1,
 
   NUMERIC = 2,
+
+  VECTOR = 3,
+};
+
+enum class VectorType : uint8_t {
+  FLOAT64 = 1,
+};
+
+enum class DistanceMetric : uint8_t {
+  L2 = 0,
+  IP = 1,
+  COSINE = 2,
+};
+
+enum class HnswLevelType : uint8_t {
+  NODE = 1,
+  EDGE = 2,
 };
 
 struct SearchKey {
@@ -95,6 +113,26 @@ struct SearchKey {
 
   void PutIndex(std::string *dst) const { PutSizedString(dst, index); }
 
+  static void PutHnswLevelType(std::string *dst, HnswLevelType type) { PutFixed8(dst, uint8_t(type)); }
+
+  void PutHnswLevelPrefix(std::string *dst, uint16_t level) const {
+    PutNamespace(dst);
+    PutType(dst, SearchSubkeyType::FIELD);
+    PutIndex(dst);
+    PutSizedString(dst, field);
+    PutFixed16(dst, level);
+  }
+
+  void PutHnswLevelNodePrefix(std::string *dst, uint16_t level) const {
+    PutHnswLevelPrefix(dst, level);
+    PutHnswLevelType(dst, HnswLevelType::NODE);
+  }
+
+  void PutHnswLevelEdgePrefix(std::string *dst, uint16_t level) const {
+    PutHnswLevelPrefix(dst, level);
+    PutHnswLevelType(dst, HnswLevelType::EDGE);
+  }
+
   std::string ConstructIndexMeta() const {
     std::string dst;
     PutNamespace(&dst);
@@ -177,6 +215,34 @@ struct SearchKey {
     PutSizedString(&dst, key);
     return dst;
   }
+
+  std::string ConstructHnswLevelNodePrefix(uint16_t level) const {
+    std::string dst;
+    PutHnswLevelNodePrefix(&dst, level);
+    return dst;
+  }
+
+  std::string ConstructHnswNode(uint16_t level, std::string_view key) const {
+    std::string dst;
+    PutHnswLevelNodePrefix(&dst, level);
+    PutSizedString(&dst, key);
+    return dst;
+  }
+
+  std::string ConstructHnswEdgeWithSingleEnd(uint16_t level, std::string_view key) const {
+    std::string dst;
+    PutHnswLevelEdgePrefix(&dst, level);
+    PutSizedString(&dst, key);
+    return dst;
+  }
+
+  std::string ConstructHnswEdge(uint16_t level, std::string_view key1, std::string_view key2) const {
+    std::string dst;
+    PutHnswLevelEdgePrefix(&dst, level);
+    PutSizedString(&dst, key1);
+    PutSizedString(&dst, key2);
+    return dst;
+  }
 };
 
 struct IndexPrefixes {
@@ -236,6 +302,8 @@ struct IndexFieldMetadata {
         return "tag";
       case IndexFieldType::NUMERIC:
         return "numeric";
+      case IndexFieldType::VECTOR:
+        return "vector";
       default:
         return "unknown";
     }
@@ -291,6 +359,98 @@ struct NumericFieldMetadata : IndexFieldMetadata {
   bool IsSortable() const override { return true; }
 };
 
+struct HnswVectorFieldMetadata : IndexFieldMetadata {
+  VectorType vector_type;
+  uint16_t dim;
+  DistanceMetric distance_metric;
+
+  uint32_t initial_cap = 500000;   // Initial vector capacity
+  uint16_t m = 16;                 // Max allowed outgoing edges per node
+  uint32_t ef_construction = 200;  // Max potential outgoing edge candidates during construction
+  uint32_t ef_runtime = 10;        // Max top candidates held during KNN search
+  double epsilon = 0.01;           // Relative factor setting search boundaries in range queries
+  uint16_t num_levels = 0;         // Number of levels in the HNSW graph
+
+  HnswVectorFieldMetadata() : IndexFieldMetadata(IndexFieldType::VECTOR) {}
+
+  bool IsSortable() const override { return true; }
+
+  void Encode(std::string *dst) const override {
+    IndexFieldMetadata::Encode(dst);
+    PutFixed8(dst, uint8_t(vector_type));
+    PutFixed16(dst, dim);
+    PutFixed8(dst, uint8_t(distance_metric));
+    PutFixed32(dst, initial_cap);
+    PutFixed16(dst, m);
+    PutFixed32(dst, ef_construction);
+    PutFixed32(dst, ef_runtime);
+    PutDouble(dst, epsilon);
+    PutFixed16(dst, num_levels);
+  }
+
+  rocksdb::Status Decode(Slice *input) override {
+    if (auto s = IndexFieldMetadata::Decode(input); !s.ok()) {
+      return s;
+    }
+
+    constexpr size_t required_size = sizeof(uint8_t) + sizeof(uint16_t) + sizeof(uint8_t) + sizeof(uint32_t) +
+                                     sizeof(uint16_t) + sizeof(uint32_t) + sizeof(uint32_t) + sizeof(uint64_t) +
+                                     sizeof(uint16_t);
+
+    if (input->size() < required_size) {
+      return rocksdb::Status::Corruption(kErrorInsufficientLength);
+    }
+
+    GetFixed8(input, (uint8_t *)(&vector_type));
+    GetFixed16(input, &dim);
+    GetFixed8(input, (uint8_t *)(&distance_metric));
+    GetFixed32(input, &initial_cap);
+    GetFixed16(input, &m);
+    GetFixed32(input, &ef_construction);
+    GetFixed32(input, &ef_runtime);
+    GetDouble(input, &epsilon);
+    GetFixed16(input, &num_levels);
+    return rocksdb::Status::OK();
+  }
+};
+
+struct HnswNodeFieldMetadata {
+  uint16_t num_neighbours;
+  std::vector<double> vector;
+
+  HnswNodeFieldMetadata() = default;
+  HnswNodeFieldMetadata(uint16_t num_neighbours, std::vector<double> vector)
+      : num_neighbours(num_neighbours), vector(std::move(vector)) {}
+
+  void Encode(std::string *dst) const {
+    PutFixed16(dst, num_neighbours);
+    PutFixed16(dst, static_cast<uint16_t>(vector.size()));
+    for (double element : vector) {
+      PutDouble(dst, element);
+    }
+  }
+
+  rocksdb::Status Decode(Slice *input) {
+    if (input->size() < 2 + 2) {
+      return rocksdb::Status::Corruption(kErrorInsufficientLength);
+    }
+    GetFixed16(input, (uint16_t *)(&num_neighbours));
+
+    uint16_t dim = 0;
+    GetFixed16(input, (uint16_t *)(&dim));
+
+    if (input->size() != dim * sizeof(double)) {
+      return rocksdb::Status::Corruption(kErrorIncorrectLength);
+    }
+    vector.resize(dim);
+
+    for (auto i = 0; i < dim; ++i) {
+      GetDouble(input, &vector[i]);
+    }
+    return rocksdb::Status::OK();
+  }
+};
+
 inline rocksdb::Status IndexFieldMetadata::Decode(Slice *input, std::unique_ptr<IndexFieldMetadata> &ptr) {
   if (input->size() < 1) {
     return rocksdb::Status::Corruption(kErrorInsufficientLength);
@@ -303,6 +463,9 @@ inline rocksdb::Status IndexFieldMetadata::Decode(Slice *input, std::unique_ptr<
     case IndexFieldType::NUMERIC:
       ptr = std::make_unique<NumericFieldMetadata>();
       break;
+    case IndexFieldType::VECTOR:
+      ptr = std::make_unique<HnswVectorFieldMetadata>();
+      break;
     default:
       return rocksdb::Status::Corruption("encountered unknown field type");
   }
