@@ -41,7 +41,7 @@ enum StatsMetricFlags {
   STATS_METRIC_COUNT
 };
 
-const int STATS_METRIC_SAMPLES = 16;  // Number of samples per metric
+constexpr int STATS_METRIC_SAMPLES = 16;  // Number of samples per metric
 
 struct CommandStat {
   std::atomic<uint64_t> calls;
