@@ -26,10 +26,11 @@
 
 namespace redis {
 
-rocksdb::Status Json::write(Slice ns_key, JsonMetadata *metadata, const JsonValue &json_val) {
+rocksdb::Status Json::write(engine::Context &ctx, Slice ns_key, JsonMetadata *metadata, const JsonValue &json_val) {
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisJson);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   auto format = storage_->GetConfig()->json_storage_format;
   metadata->format = format;
@@ -37,21 +38,22 @@ rocksdb::Status Json::write(Slice ns_key, JsonMetadata *metadata, const JsonValu
   std::string val;
   metadata->Encode(&val);
 
-  Status s;
+  Status redis_status;
   if (format == JsonStorageFormat::JSON) {
-    s = json_val.Dump(&val, storage_->GetConfig()->json_max_nesting_depth);
+    redis_status = json_val.Dump(&val, storage_->GetConfig()->json_max_nesting_depth);
   } else if (format == JsonStorageFormat::CBOR) {
-    s = json_val.DumpCBOR(&val, storage_->GetConfig()->json_max_nesting_depth);
+    redis_status = json_val.DumpCBOR(&val, storage_->GetConfig()->json_max_nesting_depth);
   } else {
     return rocksdb::Status::InvalidArgument("JSON storage format not supported");
   }
-  if (!s) {
-    return rocksdb::Status::InvalidArgument("Failed to encode JSON into storage: " + s.Msg());
+  if (!redis_status) {
+    return rocksdb::Status::InvalidArgument("Failed to encode JSON into storage: " + redis_status.Msg());
   }
 
-  batch->Put(metadata_cf_handle_, ns_key, val);
+  s = batch->Put(metadata_cf_handle_, ns_key, val);
+  if (!s.ok()) return s;
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
 rocksdb::Status Json::parse(const JsonMetadata &metadata, const Slice &json_bytes, JsonValue *value) {
@@ -70,42 +72,45 @@ rocksdb::Status Json::parse(const JsonMetadata &metadata, const Slice &json_byte
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::read(const Slice &ns_key, JsonMetadata *metadata, JsonValue *value) {
+rocksdb::Status Json::read(engine::Context &ctx, const Slice &ns_key, JsonMetadata *metadata, JsonValue *value) {
   std::string bytes;
   Slice rest;
 
-  auto s = GetMetadata(GetOptions{}, {kRedisJson}, ns_key, &bytes, metadata, &rest);
+  auto s = GetMetadata(ctx, {kRedisJson}, ns_key, &bytes, metadata, &rest);
   if (!s.ok()) return s;
 
   return parse(*metadata, rest, value);
 }
 
-rocksdb::Status Json::create(const std::string &ns_key, JsonMetadata &metadata, const std::string &value) {
+rocksdb::Status Json::create(engine::Context &ctx, const std::string &ns_key, JsonMetadata &metadata,
+                             const std::string &value) {
   auto json_res = JsonValue::FromString(value, storage_->GetConfig()->json_max_nesting_depth);
   if (!json_res) return rocksdb::Status::InvalidArgument(json_res.Msg());
   auto json_val = *std::move(json_res);
 
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::del(const Slice &ns_key) {
+rocksdb::Status Json::del(engine::Context &ctx, const Slice &ns_key) {
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisJson);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
-  batch->Delete(metadata_cf_handle_, ns_key);
+  s = batch->Delete(metadata_cf_handle_, ns_key);
+  if (!s.ok()) return s;
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status Json::Info(const std::string &user_key, JsonStorageFormat *storage_format) {
+rocksdb::Status Json::Info(engine::Context &ctx, const std::string &user_key, JsonStorageFormat *storage_format) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   std::string bytes;
   Slice rest;
   JsonMetadata metadata;
 
-  auto s = GetMetadata(GetOptions{}, {kRedisJson}, ns_key, &bytes, &metadata, &rest);
+  auto s = GetMetadata(ctx, {kRedisJson}, ns_key, &bytes, &metadata, &rest);
   if (!s.ok()) return s;
 
   *storage_format = metadata.format;
@@ -113,19 +118,20 @@ rocksdb::Status Json::Info(const std::string &user_key, JsonStorageFormat *stora
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::Set(const std::string &user_key, const std::string &path, const std::string &value) {
+rocksdb::Status Json::Set(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                          const std::string &value) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
 
   JsonMetadata metadata;
   JsonValue origin;
-  auto s = read(ns_key, &metadata, &origin);
+  auto s = read(ctx, ns_key, &metadata, &origin);
 
   if (s.IsNotFound()) {
     if (path != "$") return rocksdb::Status::InvalidArgument("new objects must be created at the root");
 
-    return create(ns_key, metadata, value);
+    return create(ctx, ns_key, metadata, value);
   }
 
   if (!s.ok()) return s;
@@ -137,15 +143,16 @@ rocksdb::Status Json::Set(const std::string &user_key, const std::string &path,
   auto set_res = origin.Set(path, std::move(new_val));
   if (!set_res) return rocksdb::Status::InvalidArgument(set_res.Msg());
 
-  return write(ns_key, &metadata, origin);
+  return write(ctx, ns_key, &metadata, origin);
 }
 
-rocksdb::Status Json::Get(const std::string &user_key, const std::vector<std::string> &paths, JsonValue *result) {
+rocksdb::Status Json::Get(engine::Context &ctx, const std::string &user_key, const std::vector<std::string> &paths,
+                          JsonValue *result) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   JsonValue res;
@@ -168,7 +175,7 @@ rocksdb::Status Json::Get(const std::string &user_key, const std::vector<std::st
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::ArrAppend(const std::string &user_key, const std::string &path,
+rocksdb::Status Json::ArrAppend(engine::Context &ctx, const std::string &user_key, const std::string &path,
                                 const std::vector<std::string> &values, Optionals<size_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
@@ -185,7 +192,7 @@ rocksdb::Status Json::ArrAppend(const std::string &user_key, const std::string &
 
   JsonMetadata metadata;
   JsonValue value;
-  auto s = read(ns_key, &metadata, &value);
+  auto s = read(ctx, ns_key, &metadata, &value);
   if (!s.ok()) return s;
 
   auto append_res = value.ArrAppend(path, append_values);
@@ -196,11 +203,11 @@ rocksdb::Status Json::ArrAppend(const std::string &user_key, const std::string &
       std::any_of(results->begin(), results->end(), [](std::optional<uint64_t> c) { return c.has_value(); });
   if (!is_write) return rocksdb::Status::OK();
 
-  return write(ns_key, &metadata, value);
+  return write(ctx, ns_key, &metadata, value);
 }
 
-rocksdb::Status Json::ArrIndex(const std::string &user_key, const std::string &path, const std::string &needle,
-                               ssize_t start, ssize_t end, Optionals<ssize_t> *results) {
+rocksdb::Status Json::ArrIndex(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                               const std::string &needle, ssize_t start, ssize_t end, Optionals<ssize_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   auto needle_res = JsonValue::FromString(needle, storage_->GetConfig()->json_max_nesting_depth);
@@ -209,7 +216,7 @@ rocksdb::Status Json::ArrIndex(const std::string &user_key, const std::string &p
 
   JsonMetadata metadata;
   JsonValue value;
-  auto s = read(ns_key, &metadata, &value);
+  auto s = read(ctx, ns_key, &metadata, &value);
   if (!s.ok()) return s;
 
   auto index_res = value.ArrIndex(path, needle_value.value, start, end);
@@ -219,12 +226,13 @@ rocksdb::Status Json::ArrIndex(const std::string &user_key, const std::string &p
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::Type(const std::string &user_key, const std::string &path, std::vector<std::string> *results) {
+rocksdb::Status Json::Type(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                           std::vector<std::string> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto res = json_val.Type(path);
@@ -234,8 +242,8 @@ rocksdb::Status Json::Type(const std::string &user_key, const std::string &path,
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::Merge(const std::string &user_key, const std::string &path, const std::string &merge_value,
-                            bool &result) {
+rocksdb::Status Json::Merge(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                            const std::string &merge_value, bool &result) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
@@ -243,12 +251,12 @@ rocksdb::Status Json::Merge(const std::string &user_key, const std::string &path
   JsonMetadata metadata;
   JsonValue json_val;
 
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
 
   if (s.IsNotFound()) {
     if (path != "$") return rocksdb::Status::InvalidArgument("new objects must be created at the root");
     result = true;
-    return create(ns_key, metadata, merge_value);
+    return create(ctx, ns_key, metadata, merge_value);
   }
 
   if (!s.ok()) return s;
@@ -262,17 +270,18 @@ rocksdb::Status Json::Merge(const std::string &user_key, const std::string &path
     return rocksdb::Status::OK();
   }
 
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::Clear(const std::string &user_key, const std::string &path, size_t *result) {
+rocksdb::Status Json::Clear(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                            size_t *result) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
 
   JsonValue json_val;
   JsonMetadata metadata;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
 
   if (!s.ok()) return s;
 
@@ -284,14 +293,15 @@ rocksdb::Status Json::Clear(const std::string &user_key, const std::string &path
     return rocksdb::Status::OK();
   }
 
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::ArrLen(const std::string &user_key, const std::string &path, Optionals<uint64_t> *results) {
+rocksdb::Status Json::ArrLen(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                             Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto len_res = json_val.ArrLen(path);
@@ -301,8 +311,9 @@ rocksdb::Status Json::ArrLen(const std::string &user_key, const std::string &pat
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::ArrInsert(const std::string &user_key, const std::string &path, const int64_t &index,
-                                const std::vector<std::string> &values, Optionals<uint64_t> *results) {
+rocksdb::Status Json::ArrInsert(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                                const int64_t &index, const std::vector<std::string> &values,
+                                Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   std::vector<jsoncons::json> insert_values;
@@ -318,7 +329,7 @@ rocksdb::Status Json::ArrInsert(const std::string &user_key, const std::string &
 
   JsonMetadata metadata;
   JsonValue value;
-  auto s = read(ns_key, &metadata, &value);
+  auto s = read(ctx, ns_key, &metadata, &value);
   if (!s.ok()) return s;
 
   auto insert_res = value.ArrInsert(path, index, insert_values);
@@ -329,27 +340,28 @@ rocksdb::Status Json::ArrInsert(const std::string &user_key, const std::string &
       std::any_of(results->begin(), results->end(), [](std::optional<uint64_t> c) { return c.has_value(); });
   if (!is_write) return rocksdb::Status::OK();
 
-  return write(ns_key, &metadata, value);
+  return write(ctx, ns_key, &metadata, value);
 }
 
-rocksdb::Status Json::Toggle(const std::string &user_key, const std::string &path, Optionals<bool> *results) {
+rocksdb::Status Json::Toggle(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                             Optionals<bool> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
 
   JsonMetadata metadata;
   JsonValue origin;
-  auto s = read(ns_key, &metadata, &origin);
+  auto s = read(ctx, ns_key, &metadata, &origin);
   if (!s.ok()) return s;
 
   auto toggle_res = origin.Toggle(path);
   if (!toggle_res) return rocksdb::Status::InvalidArgument(toggle_res.Msg());
   *results = std::move(*toggle_res);
 
-  return write(ns_key, &metadata, origin);
+  return write(ctx, ns_key, &metadata, origin);
 }
 
-rocksdb::Status Json::ArrPop(const std::string &user_key, const std::string &path, int64_t index,
+rocksdb::Status Json::ArrPop(engine::Context &ctx, const std::string &user_key, const std::string &path, int64_t index,
                              std::vector<std::optional<JsonValue>> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
@@ -357,7 +369,7 @@ rocksdb::Status Json::ArrPop(const std::string &user_key, const std::string &pat
 
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto pop_res = json_val.ArrPop(path, index);
@@ -368,15 +380,15 @@ rocksdb::Status Json::ArrPop(const std::string &user_key, const std::string &pat
                               [](const std::optional<JsonValue> &val) { return val.has_value(); });
   if (!is_write) return rocksdb::Status::OK();
 
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::ObjKeys(const std::string &user_key, const std::string &path,
+rocksdb::Status Json::ObjKeys(engine::Context &ctx, const std::string &user_key, const std::string &path,
                               Optionals<std::vector<std::string>> *keys) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
   auto keys_res = json_val.ObjKeys(path);
   if (!keys_res) return rocksdb::Status::InvalidArgument(keys_res.Msg());
@@ -385,15 +397,15 @@ rocksdb::Status Json::ObjKeys(const std::string &user_key, const std::string &pa
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::ArrTrim(const std::string &user_key, const std::string &path, int64_t start, int64_t stop,
-                              Optionals<uint64_t> *results) {
+rocksdb::Status Json::ArrTrim(engine::Context &ctx, const std::string &user_key, const std::string &path, int64_t start,
+                              int64_t stop, Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
 
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto len_res = json_val.ArrTrim(path, start, stop);
@@ -403,17 +415,17 @@ rocksdb::Status Json::ArrTrim(const std::string &user_key, const std::string &pa
   bool is_write =
       std::any_of(results->begin(), results->end(), [](const std::optional<uint64_t> &val) { return val.has_value(); });
   if (!is_write) return rocksdb::Status::OK();
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::Del(const std::string &user_key, const std::string &path, size_t *result) {
+rocksdb::Status Json::Del(engine::Context &ctx, const std::string &user_key, const std::string &path, size_t *result) {
   *result = 0;
 
   auto ns_key = AppendNamespacePrefix(user_key);
   LockGuard guard(storage_->GetLockManager(), ns_key);
   JsonValue json_val;
   JsonMetadata metadata;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
 
   if (!s.ok() && !s.IsNotFound()) return s;
   if (s.IsNotFound()) {
@@ -422,7 +434,7 @@ rocksdb::Status Json::Del(const std::string &user_key, const std::string &path,
 
   if (path == "$") {
     *result = 1;
-    return del(ns_key);
+    return del(ctx, ns_key);
   }
 
   auto res = json_val.Del(path);
@@ -432,21 +444,21 @@ rocksdb::Status Json::Del(const std::string &user_key, const std::string &path,
   if (*result == 0) {
     return rocksdb::Status::OK();
   }
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::NumIncrBy(const std::string &user_key, const std::string &path, const std::string &value,
-                                JsonValue *result) {
-  return numop(JsonValue::NumOpEnum::Incr, user_key, path, value, result);
+rocksdb::Status Json::NumIncrBy(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                                const std::string &value, JsonValue *result) {
+  return numop(ctx, JsonValue::NumOpEnum::Incr, user_key, path, value, result);
 }
 
-rocksdb::Status Json::NumMultBy(const std::string &user_key, const std::string &path, const std::string &value,
-                                JsonValue *result) {
-  return numop(JsonValue::NumOpEnum::Mul, user_key, path, value, result);
+rocksdb::Status Json::NumMultBy(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                                const std::string &value, JsonValue *result) {
+  return numop(ctx, JsonValue::NumOpEnum::Mul, user_key, path, value, result);
 }
 
-rocksdb::Status Json::numop(JsonValue::NumOpEnum op, const std::string &user_key, const std::string &path,
-                            const std::string &value, JsonValue *result) {
+rocksdb::Status Json::numop(engine::Context &ctx, JsonValue::NumOpEnum op, const std::string &user_key,
+                            const std::string &path, const std::string &value, JsonValue *result) {
   auto number_res = JsonValue::FromString(value);
   if (!number_res || !number_res.GetValue().value.is_number() || number_res.GetValue().value.is_string()) {
     return rocksdb::Status::InvalidArgument("the input value should be a number");
@@ -456,7 +468,7 @@ rocksdb::Status Json::numop(JsonValue::NumOpEnum op, const std::string &user_key
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
@@ -465,15 +477,15 @@ rocksdb::Status Json::numop(JsonValue::NumOpEnum op, const std::string &user_key
   if (!res) {
     return rocksdb::Status::InvalidArgument(res.Msg());
   }
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::StrAppend(const std::string &user_key, const std::string &path, const std::string &value,
-                                Optionals<uint64_t> *results) {
+rocksdb::Status Json::StrAppend(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                                const std::string &value, Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto append_res = json_val.StrAppend(path, value);
@@ -486,14 +498,15 @@ rocksdb::Status Json::StrAppend(const std::string &user_key, const std::string &
     return rocksdb::Status::OK();
   }
 
-  return write(ns_key, &metadata, json_val);
+  return write(ctx, ns_key, &metadata, json_val);
 }
 
-rocksdb::Status Json::StrLen(const std::string &user_key, const std::string &path, Optionals<uint64_t> *results) {
+rocksdb::Status Json::StrLen(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                             Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto str_lens = json_val.StrLen(path);
@@ -502,11 +515,12 @@ rocksdb::Status Json::StrLen(const std::string &user_key, const std::string &pat
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Json::ObjLen(const std::string &user_key, const std::string &path, Optionals<uint64_t> *results) {
+rocksdb::Status Json::ObjLen(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                             Optionals<uint64_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   JsonValue json_val;
-  auto s = read(ns_key, &metadata, &json_val);
+  auto s = read(ctx, ns_key, &metadata, &json_val);
   if (!s.ok()) return s;
 
   auto obj_lens = json_val.ObjLen(path);
@@ -515,8 +529,8 @@ rocksdb::Status Json::ObjLen(const std::string &user_key, const std::string &pat
   return rocksdb::Status::OK();
 }
 
-std::vector<rocksdb::Status> Json::MGet(const std::vector<std::string> &user_keys, const std::string &path,
-                                        std::vector<JsonValue> &results) {
+std::vector<rocksdb::Status> Json::MGet(engine::Context &ctx, const std::vector<std::string> &user_keys,
+                                        const std::string &path, std::vector<JsonValue> &results) {
   std::vector<Slice> ns_keys;
   std::vector<std::string> ns_keys_string;
   ns_keys.resize(user_keys.size());
@@ -529,7 +543,7 @@ std::vector<rocksdb::Status> Json::MGet(const std::vector<std::string> &user_key
 
   std::vector<JsonValue> json_vals;
   json_vals.resize(ns_keys.size());
-  auto statuses = readMulti(ns_keys, json_vals);
+  auto statuses = readMulti(ctx, ns_keys, json_vals);
 
   results.resize(ns_keys.size());
   for (size_t i = 0; i < ns_keys.size(); i++) {
@@ -547,8 +561,8 @@ std::vector<rocksdb::Status> Json::MGet(const std::vector<std::string> &user_key
   return statuses;
 }
 
-rocksdb::Status Json::MSet(const std::vector<std::string> &user_keys, const std::vector<std::string> &paths,
-                           const std::vector<std::string> &values) {
+rocksdb::Status Json::MSet(engine::Context &ctx, const std::vector<std::string> &user_keys,
+                           const std::vector<std::string> &paths, const std::vector<std::string> &values) {
   std::vector<std::string> ns_keys;
   ns_keys.reserve(user_keys.size());
   for (const auto &user_key : user_keys) {
@@ -559,7 +573,8 @@ rocksdb::Status Json::MSet(const std::vector<std::string> &user_keys, const std:
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisJson);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   for (size_t i = 0; i < user_keys.size(); i++) {
     auto json_res = JsonValue::FromString(values[i], storage_->GetConfig()->json_max_nesting_depth);
@@ -568,7 +583,7 @@ rocksdb::Status Json::MSet(const std::vector<std::string> &user_keys, const std:
     JsonMetadata metadata;
     JsonValue value;
 
-    if (auto s = read(ns_keys[i], &metadata, &value); s.IsNotFound()) {
+    if (auto s = read(ctx, ns_keys[i], &metadata, &value); s.IsNotFound()) {
       if (paths[i] != "$") return rocksdb::Status::InvalidArgument("new objects must be created at the root");
 
       value = *std::move(json_res);
@@ -598,20 +613,20 @@ rocksdb::Status Json::MSet(const std::vector<std::string> &user_keys, const std:
       return rocksdb::Status::InvalidArgument("Failed to encode JSON into storage: " + res.Msg());
     }
 
-    batch->Put(metadata_cf_handle_, ns_keys[i], val);
+    s = batch->Put(metadata_cf_handle_, ns_keys[i], val);
+    if (!s.ok()) return s;
   }
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-std::vector<rocksdb::Status> Json::readMulti(const std::vector<Slice> &ns_keys, std::vector<JsonValue> &values) {
-  rocksdb::ReadOptions read_options = storage_->DefaultMultiGetOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+std::vector<rocksdb::Status> Json::readMulti(engine::Context &ctx, const std::vector<Slice> &ns_keys,
+                                             std::vector<JsonValue> &values) {
+  rocksdb::ReadOptions read_options = ctx.DefaultMultiGetOptions();
 
   std::vector<rocksdb::Status> statuses(ns_keys.size());
   std::vector<rocksdb::PinnableSlice> pin_values(ns_keys.size());
-  storage_->MultiGet(read_options, metadata_cf_handle_, ns_keys.size(), ns_keys.data(), pin_values.data(),
+  storage_->MultiGet(ctx, read_options, metadata_cf_handle_, ns_keys.size(), ns_keys.data(), pin_values.data(),
                      statuses.data());
   for (size_t i = 0; i < ns_keys.size(); i++) {
     if (!statuses[i].ok()) continue;
@@ -626,18 +641,19 @@ std::vector<rocksdb::Status> Json::readMulti(const std::vector<Slice> &ns_keys,
   return statuses;
 }
 
-rocksdb::Status Json::DebugMemory(const std::string &user_key, const std::string &path, std::vector<size_t> *results) {
+rocksdb::Status Json::DebugMemory(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                                  std::vector<size_t> *results) {
   auto ns_key = AppendNamespacePrefix(user_key);
   JsonMetadata metadata;
   if (path == "$") {
     std::string bytes;
     Slice rest;
-    auto s = GetMetadata(GetOptions{}, {kRedisJson}, ns_key, &bytes, &metadata, &rest);
+    auto s = GetMetadata(ctx, {kRedisJson}, ns_key, &bytes, &metadata, &rest);
     if (!s.ok()) return s;
     results->emplace_back(rest.size());
   } else {
     JsonValue json_val;
-    auto s = read(ns_key, &metadata, &json_val);
+    auto s = read(ctx, ns_key, &metadata, &json_val);
     if (!s.ok()) return s;
     auto str_bytes = json_val.GetBytes(path, metadata.format, storage_->GetConfig()->json_max_nesting_depth);
     if (!str_bytes) return rocksdb::Status::InvalidArgument(str_bytes.Msg());
@@ -646,4 +662,18 @@ rocksdb::Status Json::DebugMemory(const std::string &user_key, const std::string
   return rocksdb::Status::OK();
 }
 
+rocksdb::Status Json::Resp(engine::Context &ctx, const std::string &user_key, const std::string &path,
+                           std::vector<std::string> *results, RESP resp) {
+  auto ns_key = AppendNamespacePrefix(user_key);
+  JsonMetadata metadata;
+  JsonValue json_val;
+  auto s = read(ctx, ns_key, &metadata, &json_val);
+  if (!s.ok()) return s;
+
+  auto json_resps = json_val.ConvertToResp(path, resp);
+  if (!json_resps) return rocksdb::Status::InvalidArgument(json_resps.Msg());
+  *results = std::move(*json_resps);
+  return rocksdb::Status::OK();
+}
+
 }  // namespace redis
