@@ -32,25 +32,27 @@
 
 namespace redis {
 
-rocksdb::Status ZSet::GetMetadata(Database::GetOptions get_options, const Slice &ns_key, ZSetMetadata *metadata) {
-  return Database::GetMetadata(get_options, {kRedisZSet}, ns_key, metadata);
+rocksdb::Status ZSet::GetMetadata(engine::Context &ctx, const Slice &ns_key, ZSetMetadata *metadata) {
+  return Database::GetMetadata(ctx, {kRedisZSet}, ns_key, metadata);
 }
 
-rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *mscores, uint64_t *added_cnt) {
+rocksdb::Status ZSet::Add(engine::Context &ctx, const Slice &user_key, ZAddFlags flags, MemberScores *mscores,
+                          uint64_t *added_cnt) {
   *added_cnt = 0;
 
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata;
-  rocksdb::Status s = GetMetadata(Database::GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok() && !s.IsNotFound()) return s;
 
   int added = 0;
   int changed = 0;
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   std::unordered_set<std::string_view> added_member_keys;
   for (auto it = mscores->rbegin(); it != mscores->rend(); ++it) {
     if (!added_member_keys.insert(it->member).second) {
@@ -59,7 +61,7 @@ rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *
     std::string member_key = InternalKey(ns_key, it->member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
     if (metadata.size > 0) {
       std::string old_score_bytes;
-      s = storage_->Get(rocksdb::ReadOptions(), member_key, &old_score_bytes);
+      s = storage_->Get(ctx, ctx.GetReadOptions(), member_key, &old_score_bytes);
       if (!s.ok() && !s.IsNotFound()) return s;
       if (s.ok()) {
         if (!s.IsNotFound() && flags.HasNX()) {
@@ -82,14 +84,17 @@ rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *
           old_score_bytes.append(it->member);
           std::string old_score_key =
               InternalKey(ns_key, old_score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-          batch->Delete(score_cf_handle_, old_score_key);
+          s = batch->Delete(score_cf_handle_, old_score_key);
+          if (!s.ok()) return s;
           std::string new_score_bytes;
           PutDouble(&new_score_bytes, it->score);
-          batch->Put(member_key, new_score_bytes);
+          s = batch->Put(member_key, new_score_bytes);
+          if (!s.ok()) return s;
           new_score_bytes.append(it->member);
           std::string new_score_key =
               InternalKey(ns_key, new_score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-          batch->Put(score_cf_handle_, new_score_key, Slice());
+          s = batch->Put(score_cf_handle_, new_score_key, Slice());
+          if (!s.ok()) return s;
           changed++;
         }
         continue;
@@ -100,10 +105,12 @@ rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *
     }
     std::string score_bytes;
     PutDouble(&score_bytes, it->score);
-    batch->Put(member_key, score_bytes);
+    s = batch->Put(member_key, score_bytes);
+    if (!s.ok()) return s;
     score_bytes.append(it->member);
     std::string score_key = InternalKey(ns_key, score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Put(score_cf_handle_, score_key, Slice());
+    s = batch->Put(score_cf_handle_, score_key, Slice());
+    if (!s.ok()) return s;
     added++;
   }
   if (added > 0) {
@@ -111,48 +118,50 @@ rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *
     metadata.size += added;
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
   if (flags.HasCH()) {
     *added_cnt += changed;
   }
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status ZSet::Card(const Slice &user_key, uint64_t *size) {
+rocksdb::Status ZSet::Card(engine::Context &ctx, const Slice &user_key, uint64_t *size) {
   *size = 0;
 
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   *size = metadata.size;
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Count(const Slice &user_key, const RangeScoreSpec &spec, uint64_t *size) {
-  return RangeByScore(user_key, spec, nullptr, size);
+rocksdb::Status ZSet::Count(engine::Context &ctx, const Slice &user_key, const RangeScoreSpec &spec, uint64_t *size) {
+  return RangeByScore(ctx, user_key, spec, nullptr, size);
 }
 
-rocksdb::Status ZSet::IncrBy(const Slice &user_key, const Slice &member, double increment, double *score) {
+rocksdb::Status ZSet::IncrBy(engine::Context &ctx, const Slice &user_key, const Slice &member, double increment,
+                             double *score) {
   uint64_t ret = 0;
   std::vector<MemberScore> mscores;
   mscores.emplace_back(MemberScore{member.ToString(), increment});
-  rocksdb::Status s = Add(user_key, ZAddFlags::Incr(), &mscores, &ret);
+  rocksdb::Status s = Add(ctx, user_key, ZAddFlags::Incr(), &mscores, &ret);
   if (!s.ok()) return s;
   *score = mscores[0].score;
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Pop(const Slice &user_key, int count, bool min, MemberScores *mscores) {
+rocksdb::Status ZSet::Pop(engine::Context &ctx, const Slice &user_key, int count, bool min, MemberScores *mscores) {
   mscores->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   if (count <= 0) return rocksdb::Status::OK();
   if (count > static_cast<int>(metadata.size)) count = static_cast<int>(metadata.size);
@@ -167,17 +176,16 @@ rocksdb::Status ZSet::Pop(const Slice &user_key, int count, bool min, MemberScor
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix_key);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_lower_bound = &lower_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, read_options, score_cf_handle_);
   iter->Seek(start_key);
   // see comment in RangeByScore()
   if (!min && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
@@ -189,8 +197,10 @@ rocksdb::Status ZSet::Pop(const Slice &user_key, int count, bool min, MemberScor
     GetDouble(&score_key, &score);
     mscores->emplace_back(MemberScore{score_key.ToString(), score});
     std::string default_cf_key = InternalKey(ns_key, score_key, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Delete(default_cf_key);
-    batch->Delete(score_cf_handle_, iter->key());
+    s = batch->Delete(default_cf_key);
+    if (!s.ok()) return s;
+    s = batch->Delete(score_cf_handle_, iter->key());
+    if (!s.ok()) return s;
     if (mscores->size() >= static_cast<unsigned>(count)) break;
   }
 
@@ -198,13 +208,14 @@ rocksdb::Status ZSet::Pop(const Slice &user_key, int count, bool min, MemberScor
     metadata.size -= mscores->size();
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &spec, MemberScores *mscores,
-                                  uint64_t *removed_cnt) {
+rocksdb::Status ZSet::RangeByRank(engine::Context &ctx, const Slice &user_key, const RangeRankSpec &spec,
+                                  MemberScores *mscores, uint64_t *removed_cnt) {
   if (mscores) mscores->clear();
 
   uint64_t cnt = 0;
@@ -216,8 +227,8 @@ rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &sp
   std::optional<LockGuard> lock_guard;
   if (spec.with_deletion) lock_guard.emplace(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   int start = spec.start;
@@ -239,15 +250,14 @@ rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &sp
       InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
   int removed_subkey = 0;
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix_key);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_lower_bound = &lower_bound;
 
   auto batch = storage_->GetWriteBatchBase();
-  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, read_options, score_cf_handle_);
   iter->Seek(start_key);
   // see comment in RangeByScore()
   if (spec.reversed && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
@@ -262,8 +272,10 @@ rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &sp
     if (count >= start) {
       if (spec.with_deletion) {
         std::string sub_key = InternalKey(ns_key, score_key, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-        batch->Delete(sub_key);
-        batch->Delete(score_cf_handle_, iter->key());
+        s = batch->Delete(sub_key);
+        if (!s.ok()) return s;
+        s = batch->Delete(score_cf_handle_, iter->key());
+        if (!s.ok()) return s;
         removed_subkey++;
       } else {
         if (mscores) mscores->emplace_back(MemberScore{score_key.ToString(), score});
@@ -277,14 +289,15 @@ rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &sp
     metadata.size -= removed_subkey;
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
-    return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
+    return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
   }
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::RangeByScore(const Slice &user_key, const RangeScoreSpec &spec, MemberScores *mscores,
-                                   uint64_t *removed_cnt) {
+rocksdb::Status ZSet::RangeByScore(engine::Context &ctx, const Slice &user_key, const RangeScoreSpec &spec,
+                                   MemberScores *mscores, uint64_t *removed_cnt) {
   if (mscores) mscores->clear();
 
   uint64_t cnt = 0;
@@ -296,7 +309,7 @@ rocksdb::Status ZSet::RangeByScore(const Slice &user_key, const RangeScoreSpec &
   std::optional<LockGuard> lock_guard;
   if (spec.with_deletion) lock_guard.emplace(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   // let's get familiar with score first:
@@ -344,19 +357,18 @@ rocksdb::Status ZSet::RangeByScore(const Slice &user_key, const RangeScoreSpec &
   std::string next_version_prefix_key =
       InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix_key);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_lower_bound = &lower_bound;
 
   int pos = 0;
-  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, read_options, score_cf_handle_);
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   if (!spec.reversed) {
     iter->Seek(start_key);
   } else {
@@ -381,8 +393,10 @@ rocksdb::Status ZSet::RangeByScore(const Slice &user_key, const RangeScoreSpec &
     if (spec.offset >= 0 && pos++ < spec.offset) continue;
     if (spec.with_deletion) {
       std::string sub_key = InternalKey(ns_key, score_key, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-      batch->Delete(sub_key);
-      batch->Delete(score_cf_handle_, iter->key());
+      s = batch->Delete(sub_key);
+      if (!s.ok()) return s;
+      s = batch->Delete(score_cf_handle_, iter->key());
+      if (!s.ok()) return s;
     } else {
       if (mscores) mscores->emplace_back(MemberScore{score_key.ToString(), score});
     }
@@ -394,14 +408,15 @@ rocksdb::Status ZSet::RangeByScore(const Slice &user_key, const RangeScoreSpec &
     metadata.size -= *removed_cnt;
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
-    return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
+    return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
   }
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::RangeByLex(const Slice &user_key, const RangeLexSpec &spec, MemberScores *mscores,
-                                 uint64_t *removed_cnt) {
+rocksdb::Status ZSet::RangeByLex(engine::Context &ctx, const Slice &user_key, const RangeLexSpec &spec,
+                                 MemberScores *mscores, uint64_t *removed_cnt) {
   if (mscores) mscores->clear();
 
   uint64_t cnt = 0;
@@ -419,7 +434,7 @@ rocksdb::Status ZSet::RangeByLex(const Slice &user_key, const RangeLexSpec &spec
     lock_guard.emplace(storage_->GetLockManager(), ns_key);
   }
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   std::string start_member = spec.reversed ? spec.max : spec.min;
@@ -428,19 +443,18 @@ rocksdb::Status ZSet::RangeByLex(const Slice &user_key, const RangeLexSpec &spec
   std::string next_version_prefix_key =
       InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix_key);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_lower_bound = &lower_bound;
 
   int pos = 0;
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, read_options);
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   if (!spec.reversed) {
     iter->Seek(start_key);
@@ -471,8 +485,10 @@ rocksdb::Status ZSet::RangeByLex(const Slice &user_key, const RangeLexSpec &spec
       std::string score_bytes = iter->value().ToString();
       score_bytes.append(member.data(), member.size());
       std::string score_key = InternalKey(ns_key, score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-      batch->Delete(score_cf_handle_, score_key);
-      batch->Delete(iter->key());
+      s = batch->Delete(score_cf_handle_, score_key);
+      if (!s.ok()) return s;
+      s = batch->Delete(iter->key());
+      if (!s.ok()) return s;
     } else {
       if (mscores) mscores->emplace_back(MemberScore{member.ToString(), DecodeDouble(iter->value().data())});
     }
@@ -484,42 +500,41 @@ rocksdb::Status ZSet::RangeByLex(const Slice &user_key, const RangeLexSpec &spec
     metadata.size -= *removed_cnt;
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
-    return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
+    return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
   }
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Score(const Slice &user_key, const Slice &member, double *score) {
+rocksdb::Status ZSet::Score(engine::Context &ctx, const Slice &user_key, const Slice &member, double *score) {
   std::string ns_key = AppendNamespacePrefix(user_key);
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
-  rocksdb::ReadOptions read_options;
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
-
   std::string score_bytes;
   std::string member_key = InternalKey(ns_key, member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  s = storage_->Get(read_options, member_key, &score_bytes);
+  s = storage_->Get(ctx, ctx.GetReadOptions(), member_key, &score_bytes);
   if (!s.ok()) return s;
   *score = DecodeDouble(score_bytes.data());
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Remove(const Slice &user_key, const std::vector<Slice> &members, uint64_t *removed_cnt) {
+rocksdb::Status ZSet::Remove(engine::Context &ctx, const Slice &user_key, const std::vector<Slice> &members,
+                             uint64_t *removed_cnt) {
   *removed_cnt = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   int removed = 0;
   std::unordered_set<std::string_view> mset;
   for (const auto &member : members) {
@@ -528,12 +543,14 @@ rocksdb::Status ZSet::Remove(const Slice &user_key, const std::vector<Slice> &me
     }
     std::string member_key = InternalKey(ns_key, member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
     std::string score_bytes;
-    s = storage_->Get(rocksdb::ReadOptions(), member_key, &score_bytes);
+    s = storage_->Get(ctx, ctx.GetReadOptions(), member_key, &score_bytes);
     if (s.ok()) {
       score_bytes.append(member.data(), member.size());
       std::string score_key = InternalKey(ns_key, score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-      batch->Delete(member_key);
-      batch->Delete(score_cf_handle_, score_key);
+      s = batch->Delete(member_key);
+      if (!s.ok()) return s;
+      s = batch->Delete(score_cf_handle_, score_key);
+      if (!s.ok()) return s;
       removed++;
     }
   }
@@ -542,27 +559,27 @@ rocksdb::Status ZSet::Remove(const Slice &user_key, const std::vector<Slice> &me
     metadata.size -= removed;
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status ZSet::Rank(const Slice &user_key, const Slice &member, bool reversed, int *member_rank,
-                           double *member_score) {
+rocksdb::Status ZSet::Rank(engine::Context &ctx, const Slice &user_key, const Slice &member, bool reversed,
+                           int *member_rank, double *member_score) {
   *member_rank = -1;
   *member_score = 0.0;
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ZSetMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   std::string score_bytes;
   std::string member_key = InternalKey(ns_key, member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  s = storage_->Get(read_options, member_key, &score_bytes);
+  s = storage_->Get(ctx, read_options, member_key, &score_bytes);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   double target_score = DecodeDouble(score_bytes.data());
@@ -581,7 +598,7 @@ rocksdb::Status ZSet::Rank(const Slice &user_key, const Slice &member, bool reve
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_lower_bound = &lower_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, read_options, score_cf_handle_);
   iter->Seek(start_key);
   // see comment in RangeByScore()
   if (reversed && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
@@ -601,42 +618,46 @@ rocksdb::Status ZSet::Rank(const Slice &user_key, const Slice &member, bool reve
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Overwrite(const Slice &user_key, const MemberScores &mscores) {
+rocksdb::Status ZSet::Overwrite(engine::Context &ctx, const Slice &user_key, const MemberScores &mscores) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ZSetMetadata metadata;
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisZSet);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   for (const auto &ms : mscores) {
     std::string score_bytes;
     std::string member_key = InternalKey(ns_key, ms.member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
     PutDouble(&score_bytes, ms.score);
-    batch->Put(member_key, score_bytes);
+    s = batch->Put(member_key, score_bytes);
+    if (!s.ok()) return s;
     score_bytes.append(ms.member);
     std::string score_key = InternalKey(ns_key, score_bytes, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Put(score_cf_handle_, score_key, Slice());
+    s = batch->Put(score_cf_handle_, score_key, Slice());
+    if (!s.ok()) return s;
   }
   metadata.size = static_cast<uint32_t>(mscores.size());
   std::string bytes;
   metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, ns_key, bytes);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+  if (!s.ok()) return s;
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status ZSet::InterStore(const Slice &dst, const std::vector<KeyWeight> &keys_weights,
+rocksdb::Status ZSet::InterStore(engine::Context &ctx, const Slice &dst, const std::vector<KeyWeight> &keys_weights,
                                  AggregateMethod aggregate_method, uint64_t *saved_cnt) {
   *saved_cnt = 0;
   std::vector<MemberScore> members;
-  auto s = Inter(keys_weights, aggregate_method, &members);
+  auto s = Inter(ctx, keys_weights, aggregate_method, &members);
   if (!s.ok()) return s;
   *saved_cnt = members.size();
-  return Overwrite(dst, members);
+  return Overwrite(ctx, dst, members);
 }
 
-rocksdb::Status ZSet::Inter(const std::vector<KeyWeight> &keys_weights, AggregateMethod aggregate_method,
-                            std::vector<MemberScore> *members) {
+rocksdb::Status ZSet::Inter(engine::Context &ctx, const std::vector<KeyWeight> &keys_weights,
+                            AggregateMethod aggregate_method, std::vector<MemberScore> *members) {
   std::vector<std::string> lock_keys;
   lock_keys.reserve(keys_weights.size());
   for (const auto &key_weight : keys_weights) {
@@ -650,7 +671,7 @@ rocksdb::Status ZSet::Inter(const std::vector<KeyWeight> &keys_weights, Aggregat
   std::vector<MemberScore> target_mscores;
   uint64_t target_size = 0;
   RangeScoreSpec spec;
-  auto s = RangeByScore(keys_weights[0].key, spec, &target_mscores, &target_size);
+  auto s = RangeByScore(ctx, keys_weights[0].key, spec, &target_mscores, &target_size);
   if (!s.ok() || target_mscores.empty()) return s;
 
   for (const auto &ms : target_mscores) {
@@ -661,7 +682,7 @@ rocksdb::Status ZSet::Inter(const std::vector<KeyWeight> &keys_weights, Aggregat
   }
 
   for (size_t i = 1; i < keys_weights.size(); i++) {
-    s = RangeByScore(keys_weights[i].key, spec, &target_mscores, &target_size);
+    s = RangeByScore(ctx, keys_weights[i].key, spec, &target_mscores, &target_size);
     if (!s.ok() || target_mscores.empty()) return s;
 
     for (const auto &ms : target_mscores) {
@@ -700,7 +721,8 @@ rocksdb::Status ZSet::Inter(const std::vector<KeyWeight> &keys_weights, Aggregat
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::InterCard(const std::vector<std::string> &user_keys, uint64_t limit, uint64_t *inter_cnt) {
+rocksdb::Status ZSet::InterCard(engine::Context &ctx, const std::vector<std::string> &user_keys, uint64_t limit,
+                                uint64_t *inter_cnt) {
   std::vector<std::string> lock_keys;
   lock_keys.reserve(user_keys.size());
   for (const auto &user_key : user_keys) {
@@ -714,7 +736,7 @@ rocksdb::Status ZSet::InterCard(const std::vector<std::string> &user_keys, uint6
   RangeScoreSpec spec;
   for (const auto &user_key : user_keys) {
     MemberScores mscores;
-    auto s = RangeByScore(user_key, spec, &mscores, nullptr);
+    auto s = RangeByScore(ctx, user_key, spec, &mscores, nullptr);
     if (!s.ok() || mscores.empty()) return s;
     mscores_list.emplace_back(std::move(mscores));
   }
@@ -746,18 +768,18 @@ rocksdb::Status ZSet::InterCard(const std::vector<std::string> &user_keys, uint6
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::UnionStore(const Slice &dst, const std::vector<KeyWeight> &keys_weights,
+rocksdb::Status ZSet::UnionStore(engine::Context &ctx, const Slice &dst, const std::vector<KeyWeight> &keys_weights,
                                  AggregateMethod aggregate_method, uint64_t *saved_cnt) {
   *saved_cnt = 0;
   std::vector<MemberScore> members;
-  auto s = Union(keys_weights, aggregate_method, &members);
+  auto s = Union(ctx, keys_weights, aggregate_method, &members);
   if (!s.ok()) return s;
   *saved_cnt = members.size();
-  return Overwrite(dst, members);
+  return Overwrite(ctx, dst, members);
 }
 
-rocksdb::Status ZSet::Union(const std::vector<KeyWeight> &keys_weights, AggregateMethod aggregate_method,
-                            std::vector<MemberScore> *members) {
+rocksdb::Status ZSet::Union(engine::Context &ctx, const std::vector<KeyWeight> &keys_weights,
+                            AggregateMethod aggregate_method, std::vector<MemberScore> *members) {
   std::vector<std::string> lock_keys;
   lock_keys.reserve(keys_weights.size());
   for (const auto &key_weight : keys_weights) {
@@ -772,7 +794,7 @@ rocksdb::Status ZSet::Union(const std::vector<KeyWeight> &keys_weights, Aggregat
   RangeScoreSpec spec;
   for (const auto &key_weight : keys_weights) {
     // get all member
-    auto s = RangeByScore(key_weight.key, spec, &target_mscores, &target_size);
+    auto s = RangeByScore(ctx, key_weight.key, spec, &target_mscores, &target_size);
     if (!s.ok() && !s.IsNotFound()) return s;
     for (const auto &ms : target_mscores) {
       double score = ms.score * key_weight.weight;
@@ -808,12 +830,12 @@ rocksdb::Status ZSet::Union(const std::vector<KeyWeight> &keys_weights, Aggregat
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::Scan(const Slice &user_key, const std::string &cursor, uint64_t limit,
+rocksdb::Status ZSet::Scan(engine::Context &ctx, const Slice &user_key, const std::string &cursor, uint64_t limit,
                            const std::string &member_prefix, std::vector<std::string> *members,
                            std::vector<double> *scores) {
   if (scores != nullptr) {
     std::vector<std::string> values;
-    auto s = SubKeyScanner::Scan(kRedisZSet, user_key, cursor, limit, member_prefix, members, &values);
+    auto s = SubKeyScanner::Scan(ctx, kRedisZSet, user_key, cursor, limit, member_prefix, members, &values);
     if (!s.ok()) return s;
 
     for (const auto &value : values) {
@@ -822,26 +844,24 @@ rocksdb::Status ZSet::Scan(const Slice &user_key, const std::string &cursor, uin
     }
     return s;
   }
-  return SubKeyScanner::Scan(kRedisZSet, user_key, cursor, limit, member_prefix, members);
+  return SubKeyScanner::Scan(ctx, kRedisZSet, user_key, cursor, limit, member_prefix, members);
 }
 
-rocksdb::Status ZSet::MGet(const Slice &user_key, const std::vector<Slice> &members,
+rocksdb::Status ZSet::MGet(engine::Context &ctx, const Slice &user_key, const std::vector<Slice> &members,
                            std::map<std::string, double> *mscores) {
   mscores->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ZSetMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
   std::string score_bytes;
   for (const auto &member : members) {
     std::string member_key = InternalKey(ns_key, member, metadata.version, storage_->IsSlotIdEncoded()).Encode();
     score_bytes.clear();
-    s = storage_->Get(read_options, member_key, &score_bytes);
+    s = storage_->Get(ctx, ctx.GetReadOptions(), member_key, &score_bytes);
     if (!s.ok() && !s.IsNotFound()) return s;
     if (s.IsNotFound()) {
       continue;
@@ -852,27 +872,27 @@ rocksdb::Status ZSet::MGet(const Slice &user_key, const std::vector<Slice> &memb
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::GetAllMemberScores(const Slice &user_key, std::vector<MemberScore> *member_scores) {
+rocksdb::Status ZSet::GetAllMemberScores(engine::Context &ctx, const Slice &user_key,
+                                         std::vector<MemberScore> *member_scores) {
   member_scores->clear();
   std::string ns_key = AppendNamespacePrefix(user_key);
   ZSetMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   std::string prefix_key = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
   std::string next_version_prefix_key =
       InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
 
   rocksdb::Slice upper_bound(next_version_prefix_key);
   rocksdb::Slice lower_bound(prefix_key);
   read_options.iterate_upper_bound = &upper_bound;
   read_options.iterate_lower_bound = &lower_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, read_options, score_cf_handle_);
 
   for (iter->Seek(prefix_key); iter->Valid() && iter->key().starts_with(prefix_key); iter->Next()) {
     InternalKey ikey(iter->key(), storage_->IsSlotIdEncoded());
@@ -885,7 +905,7 @@ rocksdb::Status ZSet::GetAllMemberScores(const Slice &user_key, std::vector<Memb
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::RandMember(const Slice &user_key, int64_t command_count,
+rocksdb::Status ZSet::RandMember(engine::Context &ctx, const Slice &user_key, int64_t command_count,
                                  std::vector<MemberScore> *member_scores) {
   if (command_count == 0) {
     return rocksdb::Status::OK();
@@ -896,24 +916,24 @@ rocksdb::Status ZSet::RandMember(const Slice &user_key, int64_t command_count,
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ZSetMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   if (metadata.size == 0) return rocksdb::Status::OK();
 
   return ExtractRandMemberFromSet<MemberScore>(
       unique, count,
-      [this, user_key](std::vector<MemberScore> *scores) -> rocksdb::Status {
-        return this->GetAllMemberScores(user_key, scores);
+      [this, user_key, &ctx](std::vector<MemberScore> *scores) -> rocksdb::Status {
+        return this->GetAllMemberScores(ctx, user_key, scores);
       },
       member_scores);
 }
 
-rocksdb::Status ZSet::Diff(const std::vector<Slice> &keys, MemberScores *members) {
+rocksdb::Status ZSet::Diff(engine::Context &ctx, const std::vector<Slice> &keys, MemberScores *members) {
   members->clear();
   MemberScores source_member_scores;
   RangeScoreSpec spec;
   uint64_t first_element_size = 0;
-  auto s = RangeByScore(keys[0], spec, &source_member_scores, &first_element_size);
+  auto s = RangeByScore(ctx, keys[0], spec, &source_member_scores, &first_element_size);
   if (!s.ok()) return s;
 
   if (first_element_size == 0) {
@@ -924,7 +944,7 @@ rocksdb::Status ZSet::Diff(const std::vector<Slice> &keys, MemberScores *members
   MemberScores target_member_scores;
   for (size_t i = 1; i < keys.size(); i++) {
     uint64_t size = 0;
-    s = RangeByScore(keys[i], spec, &target_member_scores, &size);
+    s = RangeByScore(ctx, keys[i], spec, &target_member_scores, &size);
     if (!s.ok()) return s;
     for (auto &member_score : target_member_scores) {
       exclude_members.emplace(std::move(member_score.member));
@@ -939,12 +959,13 @@ rocksdb::Status ZSet::Diff(const std::vector<Slice> &keys, MemberScores *members
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status ZSet::DiffStore(const Slice &dst, const std::vector<Slice> &keys, uint64_t *stored_count) {
+rocksdb::Status ZSet::DiffStore(engine::Context &ctx, const Slice &dst, const std::vector<Slice> &keys,
+                                uint64_t *stored_count) {
   MemberScores mscores;
-  auto s = Diff(keys, &mscores);
+  auto s = Diff(ctx, keys, &mscores);
   if (!s.ok()) return s;
   *stored_count = mscores.size();
-  return Overwrite(dst, mscores);
+  return Overwrite(ctx, dst, mscores);
 }
 
 }  // namespace redis
