@@ -56,6 +56,11 @@ struct IndexerTest : TestBase {
     auto json_info = std::make_unique<kqir::IndexInfo>("jsontest", json_field_meta, ns);
     json_info->Add(kqir::FieldInfo("$.x", std::make_unique<redis::TagFieldMetadata>()));
     json_info->Add(kqir::FieldInfo("$.y", std::make_unique<redis::NumericFieldMetadata>()));
+    auto hnsw_field_meta = std::make_unique<redis::HnswVectorFieldMetadata>();
+    hnsw_field_meta->vector_type = redis::VectorType::FLOAT64;
+    hnsw_field_meta->dim = 3;
+    hnsw_field_meta->distance_metric = redis::DistanceMetric::L2;
+    json_info->Add(kqir::FieldInfo("$.z", std::move(hnsw_field_meta)));
     json_info->prefixes.prefixes.emplace_back("idxtestjson");
 
     map.emplace("jsontest", std::move(json_info));
@@ -72,7 +77,7 @@ TEST_F(IndexerTest, HashTag) {
   auto cfhandler = storage_->GetCFHandle(ColumnFamilyID::Search);
 
   {
-    auto s = indexer.Record("no_exist", ns);
+    auto s = indexer.Record(*ctx_, "no_exist", ns);
     ASSERT_TRUE(s.Is<Status::NoPrefixMatched>());
   }
 
@@ -80,80 +85,80 @@ TEST_F(IndexerTest, HashTag) {
   auto idxname = "hashtest";
 
   {
-    auto s = indexer.Record(key1, ns);
+    auto s = indexer.Record(*ctx_, key1, ns);
     ASSERT_EQ(s.Msg(), Status::ok_msg);
     ASSERT_EQ(s->updater.info->name, idxname);
     ASSERT_TRUE(s->fields.empty());
 
     uint64_t cnt = 0;
-    db.Set(key1, "x", "food,kitChen,Beauty", &cnt);
+    db.Set(*ctx_, key1, "x", "food,kitChen,Beauty", &cnt);
     ASSERT_EQ(cnt, 1);
 
-    auto s2 = indexer.Update(*s);
+    auto s2 = indexer.Update(*ctx_, *s);
     ASSERT_TRUE(s2);
 
     auto key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("food", key1);
 
     std::string val;
-    auto s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("kitchen", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("beauty", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
   }
 
   {
-    auto s = indexer.Record(key1, ns);
+    auto s = indexer.Record(*ctx_, key1, ns);
     ASSERT_TRUE(s);
     ASSERT_EQ(s->updater.info->name, idxname);
     ASSERT_EQ(s->fields.size(), 1);
     ASSERT_EQ(s->fields["x"], T("food,kitChen,Beauty"));
 
     uint64_t cnt = 0;
-    auto s_set = db.Set(key1, "x", "Clothing,FOOD,sport", &cnt);
+    auto s_set = db.Set(*ctx_, key1, "x", "Clothing,FOOD,sport", &cnt);
     ASSERT_EQ(cnt, 0);
     ASSERT_TRUE(s_set.ok());
 
-    auto s2 = indexer.Update(*s);
+    auto s2 = indexer.Update(*ctx_, *s);
     ASSERT_TRUE(s2);
 
     auto key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("food", key1);
 
     std::string val;
-    auto s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("clothing", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("sport", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("kitchen", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.IsNotFound());
 
     key = redis::SearchKey(ns, idxname, "x").ConstructTagFieldData("beauty", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.IsNotFound());
   }
 }
@@ -163,7 +168,7 @@ TEST_F(IndexerTest, JsonTag) {
   auto cfhandler = storage_->GetCFHandle(ColumnFamilyID::Search);
 
   {
-    auto s = indexer.Record("no_exist", ns);
+    auto s = indexer.Record(*ctx_, "no_exist", ns);
     ASSERT_TRUE(s.Is<Status::NoPrefixMatched>());
   }
 
@@ -171,77 +176,77 @@ TEST_F(IndexerTest, JsonTag) {
   auto idxname = "jsontest";
 
   {
-    auto s = indexer.Record(key1, ns);
+    auto s = indexer.Record(*ctx_, key1, ns);
     ASSERT_TRUE(s);
     ASSERT_EQ(s->updater.info->name, idxname);
     ASSERT_TRUE(s->fields.empty());
 
-    auto s_set = db.Set(key1, "$", R"({"x": "food,kitChen,Beauty"})");
+    auto s_set = db.Set(*ctx_, key1, "$", R"({"x": "food,kitChen,Beauty"})");
     ASSERT_TRUE(s_set.ok());
 
-    auto s2 = indexer.Update(*s);
+    auto s2 = indexer.Update(*ctx_, *s);
     ASSERT_TRUE(s2);
 
     auto key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("food", key1);
 
     std::string val;
-    auto s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("kitchen", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("beauty", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
   }
 
   {
-    auto s = indexer.Record(key1, ns);
+    auto s = indexer.Record(*ctx_, key1, ns);
     ASSERT_TRUE(s);
     ASSERT_EQ(s->updater.info->name, idxname);
     ASSERT_EQ(s->fields.size(), 1);
     ASSERT_EQ(s->fields["$.x"], T("food,kitChen,Beauty"));
 
-    auto s_set = db.Set(key1, "$.x", "\"Clothing,FOOD,sport\"");
+    auto s_set = db.Set(*ctx_, key1, "$.x", "\"Clothing,FOOD,sport\"");
     ASSERT_TRUE(s_set.ok());
 
-    auto s2 = indexer.Update(*s);
+    auto s2 = indexer.Update(*ctx_, *s);
     ASSERT_TRUE(s2);
 
     auto key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("food", key1);
 
     std::string val;
-    auto s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("clothing", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("sport", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("kitchen", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.IsNotFound());
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("beauty", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.IsNotFound());
   }
 }
@@ -254,29 +259,68 @@ TEST_F(IndexerTest, JsonTagBuildIndex) {
   auto idxname = "jsontest";
 
   {
-    auto s_set = db.Set(key1, "$", R"({"x": "food,kitChen,Beauty"})");
+    auto s_set = db.Set(*ctx_, key1, "$", R"({"x": "food,kitChen,Beauty"})");
     ASSERT_TRUE(s_set.ok());
 
-    auto s2 = indexer.updater_list[1].Build();
+    auto s2 = indexer.updater_list[1].Build(*ctx_);
     ASSERT_EQ(s2.Msg(), Status::ok_msg);
 
     auto key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("food", key1);
 
     std::string val;
-    auto s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("kitchen", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
 
     key = redis::SearchKey(ns, idxname, "$.x").ConstructTagFieldData("beauty", key1);
 
-    s3 = storage_->Get(storage_->DefaultMultiGetOptions(), cfhandler, key, &val);
+    s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, key, &val);
     ASSERT_TRUE(s3.ok());
     ASSERT_EQ(val, "");
   }
 }
+
+TEST_F(IndexerTest, JsonHnswVector) {
+  redis::Json db(storage_.get(), ns);
+  auto cfhandler = storage_->GetCFHandle(ColumnFamilyID::Search);
+
+  {
+    auto s = indexer.Record(*ctx_, "no_exist", ns);
+    ASSERT_TRUE(s.Is<Status::NoPrefixMatched>());
+  }
+
+  auto key3 = "idxtestjson:k3";
+  auto idxname = "jsontest";
+
+  {
+    auto s = indexer.Record(*ctx_, key3, ns);
+    ASSERT_TRUE(s);
+    ASSERT_EQ(s->updater.info->name, idxname);
+    ASSERT_TRUE(s->fields.empty());
+
+    auto s_set = db.Set(*ctx_, key3, "$", R"({"z": [1,2,3]})");
+    ASSERT_TRUE(s_set.ok());
+
+    auto s2 = indexer.Update(*ctx_, *s);
+    EXPECT_EQ(s2.Msg(), Status::ok_msg);
+
+    auto search_key = redis::SearchKey(ns, idxname, "$.z").ConstructHnswNode(0, key3);
+
+    std::string val;
+    auto s3 = storage_->Get(*ctx_, ctx_->DefaultMultiGetOptions(), cfhandler, search_key, &val);
+    ASSERT_TRUE(s3.ok());
+
+    redis::HnswNodeFieldMetadata node_meta;
+    Slice input(val);
+    node_meta.Decode(&input);
+    EXPECT_EQ(node_meta.num_neighbours, 0);
+    std::vector<double> expected = {1, 2, 3};
+    EXPECT_EQ(expected, node_meta.vector);
+  }
+}
